#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Schedule 497, the Late Contribution Reports.

More about the filing: http://calaccess.californiacivicdata.org/documentation/calaccess-forms/f497/
"""
from django.db import models
from .base import Form497ItemBase


class Form497Part2ItemBase(Form497ItemBase):
    """
    Abstract base model for items reported on Part 2 of Schedule 497 filings.

    On Part 2 of Schedule 497, campaign filers are required to report
    contributions made in the 90 days leading up to an election.
    """
    RECIPIENT_CODE_CHOICES = (
        ('BNM', 'Ballot measure name/title'),
        ('CAO', 'Candidate/officeholder'),
        ('COM', 'Committee'),
        ('CTL', 'Controlled committee'),
        ('IND', 'Individual'),
        ('OTH', 'Other'),
        ('PTY', 'Political Party'),
        ('RCP', 'Recipient committee'),
        ('SCC', 'Small Contributor Committee'),
    )
    recipient_code = models.CharField(
        verbose_name='recipient code',
        max_length=3,
        blank=True,
        choices=RECIPIENT_CODE_CHOICES,
        help_text='Code describing the recipient (from S497_CD.ENTITY_CD)',
    )
    recipient_committee_id = models.CharField(
        verbose_name='recipient committee id',
        max_length=9,
        blank=True,
        help_text='Filer identification number identifying the recipient if it '
                  'is a committee (from S497_CD.CMTE_ID)'
    )
    recipient_title = models.CharField(
        verbose_name='recipient title',
        max_length=10,
        blank=True,
        help_text='Name title of the recipient (from S497_CD.ENTY_NAMT)',
    )
    recipient_lastname = models.CharField(
        verbose_name='recipient lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the recipient (from S497_CD.ENTY_NAML)',
    )
    recipient_firstname = models.CharField(
        verbose_name='recipient firstname',
        max_length=45,
        help_text='First name of the recipient (from S497_CD.ENTY_NAMF)',
    )
    recipient_name_suffix = models.CharField(
        verbose_name='recipient name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the recipient (from S497_CD.ENTY_NAMS)',
    )
    recipient_city = models.CharField(
        verbose_name='recipient city',
        max_length=30,
        blank=True,
        help_text='City of the recipient (from S497_CD.ENTY_CITY)',
    )
    recipient_state = models.CharField(
        verbose_name='recipient state',
        max_length=2,
        blank=True,
        help_text='State of the recipient (from S497_CD.ENTY_ST)',
    )
    recipient_zip = models.CharField(
        verbose_name='recipient zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'recipient (from S497_CD.ENTY_ZIP4)',
    )
    candidate_id = models.CharField(
        verbose_name='candidate id',
        max_length=9,
        blank=True,
        help_text='Identifies the candidate to whom the contribution is '
                  'connected (from S497_CD.CAND_ID). This can be translated '
                  'to the filer_id by joining to FILER_XREF_CD.',
    )
    candidate_title = models.CharField(
        verbose_name='candidate title',
        max_length=10,
        blank=True,
        help_text='Name title of the candidate to whom the contribution is '
                  'connected (from S497_CD.CAND_NAMT)',
    )
    candidate_lastname = models.CharField(
        verbose_name='candidate last name',
        max_length=200,
        blank=True,
        help_text='Last name of the candidate to whom the contribution is '
                  'connected (S497_CD.CAND_NAML)',
    )
    candidate_firstname = models.CharField(
        verbose_name='candidate first name',
        max_length=45,
        blank=True,
        help_text='First name of the candidate to whom the contribution is '
                  'connected (S497_CD.CAND_NAMF)',
    )
    candidate_namesuffix = models.CharField(
        verbose_name='candidate name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the candidate to whom the contribution is '
                  'connected (S497_CD.CAND_NAMS)',
    )
    candidate_office_code = models.CharField(
        verbose_name='candidate office code',
        max_length=3,
        blank=True,
        help_text='Code describing the office sought sought by the candidate '
                  '(from S497_CD.OFFICE_CD)',
    )
    candidate_office_description = models.CharField(
        verbose_name='candidate office description',
        max_length=40,
        blank=True,
        help_text='Full description of the office sought by the candidate '
                  '(from S497_CD.OFFIC_DSCR)',
    )
    candidate_jurisdiction_code = models.CharField(
        verbose_name='candidate jursidiction code',
        max_length=3,
        blank=True,
        help_text='Code describing the jurisdiction of office sought by the '
                  'candidate (from S497_CD.JURIS_CD)',
    )
    candidate_jurisdiction_description = models.CharField(
        verbose_name='candidate jurisdiction description',
        max_length=40,
        blank=True,
        help_text='Full description of the jurisdiction of the office sought '
                  'by the candidate (from S497_CD.JURIS_DSCR)',
    )
    candidate_district = models.CharField(
        verbose_name='candidate district',
        max_length=3,
        blank=True,
        help_text='District of the office sought by the candidate (from '
                  'S497_CD.DIST_NO)',
    )
    ballot_measure_name = models.CharField(
        verbose_name='Ballot measure name',
        max_length=200,
        blank=True,
        help_text='Name of the ballot measure supported or opposed by the '
                  'recipient (from S497_CD.BAL_NAME)',
    )
    ballot_measure_number = models.CharField(
        verbose_name='Ballot measure number',
        max_length=7,
        blank=True,
        help_text='Ballot measure number (from S497_CD.BAL_NUM)',
    )
    ballot_measure_jurisdiction = models.CharField(
        verbose_name='ballot measure jurisdiction',
        max_length=40,
        blank=True,
        help_text='Jurisdiction of the ballot measure supported or opposed by '
                  'the recipient (from S497_CD.BAL_JURIS)',
    )
    SUPPORT_OPPOSITION_CODE_CHOICES = (
        ('O', 'Opposition'),
        ('S', 'Support'),
    )
    support_opposition_code = models.CharField(
        verbose_name='support opposition code',
        max_length=1,
        choices=SUPPORT_OPPOSITION_CODE_CHOICES,
        blank=True,
        help_text='Code describing whether the contribuitor supports or opposes'
                  'the candidate or ballot measure (from S497_CD.SUP_OPP_CD)',
    )
    election_date = models.DateField(
        verbose_name='election date',
        null=True,
        help_text='Date of the election (from S497_CD.ELEC_DATE)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form497Part2Item(Form497Part2ItemBase):
    """
    Late contributions made by campaign filers.

    These transactions are itemized on Part 2 of the most recent version
    of each Schedule 497 filing. For gifts of late contributions itemized on
    any version of any Schedule 497 filing, see Form497Part2ItemVersion.

    Derived from S497_CD records where FORM_TYPE is 'F497P2'.
    """
    filing = models.ForeignKey(
        'Form497Filing',
        related_name='contributions_made',
        db_constraint=False,
        null=True,
        on_delete=models.SET_NULL,
        help_text='Unique identification number for the Schedule 497 filing ('
                  'from S497_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 497 (Late Contribution) Part 2 item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form497Part2ItemVersion(Form497Part2ItemBase):
    """
    Every version of each late contribution made by a campaign filer.

    For late contributions itemized on Part 2 of the most recent version of
    each Schedule 497 filing, see Form497Part2Item.

    Derived from S497_CD records where FORM_TYPE is 'F497P2'.
    """
    filing_version = models.ForeignKey(
        'Form497FilingVersion',
        related_name='contributions_made',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Schedule 497 '
                  'that includes the given contribution'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 497 (Late Contribution) Part 2 item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals

# Models
from calaccess_processed.proxies import OCDProxyModelMixin
from opencivicdata.core.models import Post

# Managers
from calaccess_processed_elections.managers import (
    OCDPostManager,
    OCDAssemblyPostManager,
    OCDExecutivePostManager,
    OCDSenatePostManager
)


class OCDPostProxy(Post, OCDProxyModelMixin):
    """
    A proxy on the OCD Post model with helper methods..
    """
    objects = OCDPostManager()
    assembly = OCDAssemblyPostManager()
    executive = OCDExecutivePostManager()
    senate = OCDSenatePostManager()

    copy_to_fields = (
        ('id',),
        ('label',),
        ('role',),
        ('organization_id',),
        ('division_id',),
        ('start_date',),
        ('end_date',),
        ('maximum_memberships',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load the OCD CandidateContest and related models with scraped CAL-ACCESS data.
"""
from calaccess_processed_elections.proxies import (
    OCDCandidateContestProxy,
    OCDCandidacyProxy,
    ScrapedCandidateProxy
)
from calaccess_processed.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Load the OCD CandidateContest and related models with scraped CAL-ACCESS data.
    """
    help = 'Load the OCD CandidateContest and related models with scraped CAL-ACCESS data'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header("Loading Candidate Contests")
        if self.verbosity > 2:
            self.log(" ...with CAL-ACCESS filer_ids")
            self.load_candidates_with_filer_ids()
        if self.verbosity > 2:
            self.log(" ...without CAL-ACCESS filer_ids")
            self.load_candidates_without_filer_ids()

        # connect runoffs to their previously undecided contests
        if self.verbosity > 2:
            self.log(' Linking runoffs to previous contests')
        OCDCandidateContestProxy.objects.set_parents()

        self.success("Done!")

    def load_candidates_with_filer_ids(self):
        """
        Load scraped candidates that were collected with a filer_id.
        """
        scraped_candidates = ScrapedCandidateProxy.objects.exclude(scraped_id='')

        for scraped_candidate in scraped_candidates:
            if self.verbosity > 2:
                self.log(f" Loading scraped candidate: {scraped_candidate}")
            candidacy = self.load_scraped_candidate(scraped_candidate)

            form501s = scraped_candidate.match_form501s_by_scraped_id()

            if form501s.exists():
                for form501 in form501s:
                    candidacy.link_form501(form501.filing_id)
                candidacy.update_from_form501()
                candidacy.link_filer_ids_from_form501s()

            self.correct_candidacy_party(scraped_candidate, candidacy)

    def load_candidates_without_filer_ids(self):
        """
        Load scraped candidates that were collected without a filer_id.

        These are the candidates on the state pages that are text only.

        They tend to be the least significant candidates, since they typically have never created a committee.
        """
        # Jim Fitzgerald's independent run for Senate 15 in 2008.
        # He was on the ballot in the general but records and a phone
        # interview with the candidate show he did not run in any primary.
        scraped_candidates = ScrapedCandidateProxy.objects.filter(
            scraped_id=''
        ).exclude(
            name__contains='FITZGERALD',
            election__name='2008 PRIMARY',
            office_name='STATE SENATE 15'
        )

        for scraped_candidate in scraped_candidates:
            if self.verbosity > 2:
                self.log(f" Loading scraped candidate: {scraped_candidate}")
            candidacy = self.load_scraped_candidate(scraped_candidate)

            form501s = scraped_candidate.match_form501s_by_name()

            if form501s.exists():
                for form501 in form501s:
                    candidacy.link_form501(form501.filing_id)
                candidacy.update_from_form501()
                candidacy.link_filer_ids_from_form501s()

            self.correct_candidacy_party(scraped_candidate, candidacy)

    def load_scraped_candidate(self, scraped_candidate):
        """
        Load a scraped_candidate into OCD Candidacy and related models.

        Returns a candidacy record.
        """
        # check if this scraped candidate was previously loaded into OCD
        try:
            candidacy = scraped_candidate.get_loaded_ocd_candidacy()
        except OCDCandidacyProxy.DoesNotExist:
            # Get contest
            contest, contest_created = scraped_candidate.get_or_create_contest()

            # Create candidacy
            candidacy, candidacy_created = OCDCandidacyProxy.objects.get_or_create_from_calaccess(
                contest,
                scraped_candidate.parsed_name,
                candidate_status='qualified',
                candidate_filer_id=scraped_candidate.scraped_id or None
            )
            if candidacy_created and self.verbosity > 1:
                self.log(' Created Candidacy: %s' % candidacy)
        else:
            # Get contest
            contest, contest_created = scraped_candidate.get_or_create_contest()
            # check if the candidacy is not part of the correct contest
            if contest != candidacy.contest:
                old_contest = candidacy.contest
                if self.verbosity > 2:
                    msg = ' Resetting {0} contest: {1} -> {2}'.format(
                        candidacy,
                        old_contest,
                        contest,
                    )
                    self.log(msg)
                candidacy.contest = contest
                candidacy.save()
                # if there aren't any candidacies linked to the old contest
                # delete it
                if old_contest.candidacies.count() == 0:
                    old_contest.delete()
                    if self.verbosity > 2:
                        self.log(' Deleting empty %s' % old_contest)

        # always update the source for the candidacy
        candidacy.sources.update_or_create(
            url=scraped_candidate.url,
            note='Last scraped on {dt:%Y-%m-%d}'.format(
                dt=scraped_candidate.last_modified,
            )
        )
        return candidacy

    def correct_candidacy_party(self, scraped_candidate, candidacy):
        """
        Correct the party of the candidacy, if necessary.
        """
        # Set candidacy party
        corrected_party = scraped_candidate.get_party()
        if corrected_party:
            # if not already set
            if not candidacy.party:
                candidacy.party = corrected_party
                candidacy.save()
            # or if correction is different
            elif candidacy.party.id != corrected_party.id:
                if self.verbosity > 2:
                    msg = ' Resetting {0} party: {1} -> {2}'.format(
                        candidacy,
                        candidacy.party,
                        corrected_party,
                    )
                    self.log(msg)
                candidacy.party = corrected_party
                candidacy.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom managers for the OCD Election model.
"""
from __future__ import unicode_literals
from calaccess_processed.managers import BulkLoadSQLManager


class OCDPartisanPrimaryManager(BulkLoadSQLManager):
    """
    Custom manager for limiting OCD elections querysets to partisan primaries.
    """
    def get_queryset(self):
        """
        Returns whether or not this was a primary election held in the partisan era prior to 2012.
        """
        return super(OCDPartisanPrimaryManager, self).get_queryset().filter(
            date__year__lt=2012,
            name__icontains='PRIMARY'
        )


class OCDElectionManager(BulkLoadSQLManager):
    """
    Custom helpers for the OCD Election model.
    """
    def create_from_calaccess(self, name, dt, election_id=None, election_type=None):
        """
        Create an OCD Election object.
        """
        from calaccess_processed_elections.proxies import OCDOrganizationProxy, OCDDivisionProxy

        # Create the object
        obj = self.get_queryset().create(
            name=name,
            date=dt,
            administrative_organization=OCDOrganizationProxy.objects.elections_division(),
            division=OCDDivisionProxy.objects.california(),
        )

        # And add the identifier so we can find it in the future
        if election_id:
            obj.identifiers.create(scheme='calaccess_election_id', identifier=election_id)

        # Add the election type so we can pull it out later if we want it.
        if election_type:
            obj.extras['calaccess_election_type'] = [election_type]
            obj.save()

        # Pass it back
        return obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy model for augmenting ScrapedProposition model with methods useful for processing.
"""
from __future__ import unicode_literals
from calaccess_scraped.models import Proposition
from .propositionelections import ScrapedPropositionElectionProxy
from calaccess_processed_elections.managers import (
    ScrapedBallotMeasureManager,
    ScrapedRecallMeasureManager
)


class ScrapedPropositionProxy(Proposition):
    """
    A proxy for the Proposition model in calaccess_scraped.
    """
    ballot_measures = ScrapedBallotMeasureManager()
    recall_measures = ScrapedRecallMeasureManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    @property
    def election_proxy(self):
        """
        Return the proxy model for the related election object.
        """
        return ScrapedPropositionElectionProxy.objects.get(id=self.election.id)

    @property
    def classification(self):
        """
        Clean up the type of proposition this is for standardizing in OCD models.
        """
        # Set the classification
        if 'REFERENDUM' in self.name:
            return 'referendum'
        elif ('INITIATIVE' in self.name or 'INITATIVE' in self.name):
            return 'initiative'
        else:
            return 'ballot measure'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing Part 1 data from Schedule 497, the Late Independent Expenditure Reports.

More about the filing: https://calaccess.californiacivicdata.org/documentation/calaccess-forms/f496/
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel


class Form496Part1ItemBase(FilingBaseModel):
    """
    Abstract base model for items reported on Part 1 of Schedule 496 filings.

    On Part 1 of Schedule 496, campaign filers are required to report the candidates or ballot measure
    they are supported or opposing with their independent expenditures.
    """
    candidate_title = models.CharField(
        verbose_name='candidate title',
        max_length=10,
        blank=True,
        help_text='Name title of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMT)',
    )
    candidate_lastname = models.CharField(
        verbose_name='candidate lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAML)',
    )
    candidate_firstname = models.CharField(
        verbose_name='candidate firstname',
        max_length=45,
        help_text='First name of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMF)',
    )
    candidate_name_suffix = models.CharField(
        verbose_name='candidate name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the candidate (from CVR_CAMPAIGN_DISCLOSURE_CD.CAND_NAMS)',
    )
    candidate_id = models.CharField(
        verbose_name='candidate id',
        max_length=9,
        blank=True,
        help_text='Identifies the candidate to whom the contribution is '
                  'connected (from S497_CD.CAND_ID). This can be translated '
                  'to the filer_id by joining to FILER_XREF_CD.',
    )
    candidate_office_code = models.CharField(
        verbose_name='candidate office code',
        max_length=3,
        blank=True,
        help_text='Code describing the office sought sought by the candidate '
                  '(from CVR_CAMPAIGN_DISCLOSURE_CD.OFFICE_CD)',
    )
    ballot_measure_name = models.CharField(
        verbose_name='Ballot measure name',
        max_length=200,
        blank=True,
        help_text='Name of the ballot measure supported or opposed by the '
                  'recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NAME)',
    )
    ballot_measure_number = models.CharField(
        verbose_name='Ballot measure number',
        max_length=7,
        blank=True,
        help_text='Ballot measure number (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_NUM)',
    )
    ballot_measure_jurisdiction = models.CharField(
        verbose_name='ballot measure jurisdiction',
        max_length=40,
        blank=True,
        help_text='Jurisdiction of the ballot measure supported or opposed by '
                  'the recipient (from CVR_CAMPAIGN_DISCLOSURE_CD.BAL_JURIS)',
    )
    SUPPORT_OPPOSITION_CODE_CHOICES = (
        ('O', 'Opposition'),
        ('S', 'Support'),
    )
    support_opposition_code = models.CharField(
        verbose_name='support opposition code',
        max_length=1,
        choices=SUPPORT_OPPOSITION_CODE_CHOICES,
        blank=True,
        help_text='Code describing whether the contribuitor supports or opposes'
                  'the candidate or ballot measure (from CVR_CAMPAIGN_DISCLOSURE_CD.SUP_OPP_CD)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form496Part1Item(Form496Part1ItemBase):
    """
    Late independent expenditures made by campaign filers.

    These transactions are itemized on Part 1 of the most recent version
    of each Schedule 496 filing.
    """
    filing = models.ForeignKey(
        'Form496Filing',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Unique identification number for the Schedule 496 filing (from S496_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 496 (Late Independent Expenditure) Part 1 item"

    def __str__(self):
        return str(self.filing)


class Form496Part1ItemVersion(Form496Part1ItemBase):
    """
    Every version of each late independent expenditures made by campaign filers.
    """
    filing_version = models.ForeignKey(
        'Form496FilingVersion',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Schedule 496 that includes the received contribution'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 496 (Late Independent Expenditure) Part 1 item version"

    def __str__(self):
        return '%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the managers from submodules and thread them together.
"""
from .core import (
    OCDAssemblyDivisionManager,
    OCDSenateDivisionManager,
    OCDCaliforniaDivisionManager,
    OCDJurisdictionManager,
    OCDOrganizationManager,
    OCDMembershipManager,
    OCDPersonManager,
    OCDPostManager,
    OCDAssemblyPostManager,
    OCDExecutivePostManager,
    OCDSenatePostManager
)
from .elections import (
    OCDCandidacyQuerySet,
    OCDCandidacyManager,
    OCDCandidateContestQuerySet,
    OCDCandidateContestManager,
    OCDPartisanPrimaryManager,
    OCDElectionManager,
    OCDPartyManager
)


__all__ = (
    "OCDAssemblyDivisionManager",
    "OCDSenateDivisionManager",
    "OCDCaliforniaDivisionManager",
    "OCDJurisdictionManager",
    "OCDOrganizationManager",
    "OCDMembershipManager",
    "OCDPersonManager",
    "OCDPostManager",
    "OCDAssemblyPostManager",
    "OCDExecutivePostManager",
    "OCDSenatePostManager",
    "OCDCandidacyQuerySet",
    "OCDCandidacyManager",
    "OCDCandidateContestQuerySet",
    "OCDCandidateContestManager",
    "OCDPartisanPrimaryManager",
    "OCDElectionManager",
    "OCDPartyManager"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy model for augmenting ScrapedPropositionElection model with methods useful for processing.
"""
from __future__ import unicode_literals
import re
from django.utils import timezone
from .base import ScrapedElectionProxyMixin
from calaccess_scraped.models import PropositionElection
from calaccess_processed_elections.proxies import OCDElectionProxy


class ScrapedPropositionElectionProxy(ScrapedElectionProxyMixin, PropositionElection):
    """
    A proxy for the PropositionElection model in calaccess_scraped.
    """
    NAME_PATTERN = re.compile(r'^(?P<date>^[A-Z]+\s\d{1,2},\s\d{4}),?\s(?P<type>.+)$')

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    @property
    def election_type(self):
        """
        Return the scraped incumbent election's type.

        (e.g., "GENERAL", "PRIMARY", "SPECIAL ELECTION", "RECALL")
        """
        # Extract the name and date from the election name
        match = self.NAME_PATTERN.match(self.name)

        return match.groupdict()['type'].upper()

    @property
    def date(self):
        """
        Parse date from scraped proposition election's name.

        Return a timezone aware date object, if found, else None.
        """
        # Extract the name and date from the election name
        match = self.NAME_PATTERN.match(self.name)

        # Convert it to a datetime object
        return timezone.datetime.strptime(match.groupdict()['date'], '%B %d, %Y').date()

    def get_ocd_election(self):
        """
        Returns an OCD Election object for this record, if it exists.
        """
        try:
            ocd_election = OCDElectionProxy.objects.get(
                name=self.ocd_name,
                date=self.date,
            )
        except OCDElectionProxy.DoesNotExist:
            # If that doesn't exist, try getting it by date
            try:
                ocd_election = OCDElectionProxy.objects.get(date=self.date)
            except (
                OCDElectionProxy.DoesNotExist,
                OCDElectionProxy.MultipleObjectsReturned
            ):
                raise

        return ocd_election



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel
from calaccess_processed_filings.models.campaign import CampaignLoanReceivedItemBase


class Form460ScheduleB1ItemBase(CampaignLoanReceivedItemBase):
    """
    Abstract base model for items reported on Schedule B, Part 1, of Form 460.

    On Schedule B, Part 1, campaign filers are required to report loans
    received or outstanding during the period covered by the filing.
    """
    begin_period_balance = models.DecimalField(
        verbose_name='beginning period balance',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance of the loan at the beginning of the"
                  "period covered by the filing (from LOAN_CD.LOAN_AMT4)"
    )
    amount_received = models.DecimalField(
        verbose_name='amount received',
        decimal_places=2,
        max_digits=14,
        help_text="Amount received during the period covered by the filing "
                  "(from LOAN_CD.LOAN_AMT1)"
    )
    amount_paid = models.DecimalField(
        verbose_name='amount paid',
        decimal_places=2,
        max_digits=14,
        help_text="Amount paid back during the period covered by the filing "
                  "(from LOAN_CD.LOAN_AMT5)"
    )
    amount_forgiven = models.DecimalField(
        verbose_name='amount forgiven',
        decimal_places=2,
        max_digits=14,
        help_text="Amount forgiven by the lender during the period covered by "
                  "the filing (from LOAN_CD.LOAN_AMT6)"
    )
    end_period_balance = models.DecimalField(
        verbose_name='end period balance',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance of the loan at the end of the period "
                  "covered by the filing (from LOAN_CD.LOAN_AMT2)"
    )
    date_due = models.DateField(
        verbose_name='date due',
        null=True,
        help_text="Date that the loan is due (from LOAN_CD.LOAN_DATE2)"
    )
    interest_paid = models.DecimalField(
        verbose_name='interest paid',
        decimal_places=2,
        max_digits=14,
        help_text="Amount of interest paid on the loan during the period "
                  "covered by the campaign filing (from LOAN_CD.LOAN_AMT7)"
    )
    original_amount = models.DecimalField(
        verbose_name='original amount',
        decimal_places=2,
        max_digits=14,
        help_text="Original amount loaned by the lender to the campaign filer "
                  "(from LOAN_CD.LOAN_AMT8)"
    )
    date_incurred = models.DateField(
        verbose_name='date incurred',
        null=True,
        help_text="Date the loan was made or received (from LOAN_CD.LOAN_DATE1)"
    )
    cumulative_ytd_contributions = models.DecimalField(
        verbose_name='cumulative year-to-date contributions',
        decimal_places=2,
        max_digits=14,
        help_text="Cumulative amount of contributions (loans, monetary and "
                  "nonmonetary contributions) received from the lender during "
                  "the calendar year covered by this statement (from LOAN_CD."
                  "LOAN_AMT3)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleB1Item(Form460ScheduleB1ItemBase):
    """
    Loans received and loan payments by campaign filers.

    These transactions are itemized on Schedule B, Part 1, of the most recent
    version of each Form 460 filing. For loans itemized on any version of any
    Form 460 filing, see Form460ScheduleB1ItemVersion.

    Derived from LOAN_CD records where FORM_TYPE is 'B1'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_b1_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the loan '
                  'was reported (from LOAN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleB1ItemVersion(Form460ScheduleB1ItemBase):
    """
    Every version of each loan received or loan payment made by a campaign filer.

    For outstanding loans itemized on Schedule B, Part 1, of the most recent
    version of each Form 460 filing, see Form460ScheduleB1Item.

    Derived from LOAN_CD records where FORM_TYPE is 'B1'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_b1_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the outstanding loan'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )


class Form460ScheduleB2ItemBase(FilingBaseModel):
    """
    Abstract base model for items reported on Schedule B, Part 2, of Form 460.

    On Schedule B, Part 2, campaign filers are required to report loan
    guarantors, A "guarantor" is a third party that co-signs, endorses, or
    provides security for a loan, or establishes or provides security for a
    line of credit. A guarantor is also making a contribution.
    """
    line_item = models.IntegerField(
        verbose_name='line item',
        help_text="Line number of the filing form where the loan is "
                  "itemized (from LOAN_CD.LINE_ITEM)",
    )
    GUARANTOR_CODE_CHOICES = (
        ('COM', "Committee"),
        ('IND', "Individual"),
        ('OTH', "Other"),
        ('PTY', "Political Party"),
        ('SCC', "Small Contributor Committee"),
        ('???', "Unknown value"),
    )
    guarantor_code = models.CharField(
        verbose_name='lender code',
        max_length=3,
        blank=True,
        choices=GUARANTOR_CODE_CHOICES,
        help_text='Code describing the guarantor (from LOAN_CD.ENTITY_CD)',
    )
    guarantor_title = models.CharField(
        verbose_name='guarantor title',
        max_length=10,
        blank=True,
        help_text="Name title of the guarantor (from LOAN_CD.LNDR_NAMT)",
    )
    guarantor_lastname = models.CharField(
        verbose_name='guarantor lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the guarantor or business name (from LOAN_CD."
                  "LNDR_NAML)",
    )
    guarantor_firstname = models.CharField(
        verbose_name='guarantor firstname',
        max_length=45,
        blank=True,
        help_text="First name of the guarantor (from LOAN_CD.LNDR_NAMF)",
    )
    guarantor_name_suffix = models.CharField(
        verbose_name='guarantor name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the guarantor (from LOAN_CD.LNDR_NAMS)",
    )
    guarantor_city = models.CharField(
        verbose_name='guarantor city',
        max_length=30,
        blank=True,
        help_text='City of the guarantor (from LOAN_CD.LOAN_CITY)',
    )
    guarantor_state = models.CharField(
        verbose_name='guarantor state',
        max_length=2,
        blank=True,
        help_text='State of the guarantor (from LOAN_CD.LOAN_ST)',
    )
    guarantor_zip = models.CharField(
        verbose_name='guarantor zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'guarantor (from LOAN_CD.LOAN_ZIP4)',
    )
    guarantor_employer = models.CharField(
        verbose_name='guarantor employer',
        max_length=200,
        blank=True,
        help_text='Employer of the guarantor (from LOAN_CD.LOAN_EMP)',
    )
    guarantor_occupation = models.CharField(
        verbose_name='guarantor occupation',
        max_length=60,
        blank=True,
        help_text='Occupation of the guarantor (from LOAN_CD.LOAN_OCC)',
    )
    guarantor_is_self_employed = models.BooleanField(
        verbose_name='guarantor is self employed',
        default=False,
        help_text='Indicates whether or not the guarantor is self-employed'
                  '(from LOAN_CD.LOAN_SELF)',
    )
    lender_name = models.CharField(
        verbose_name='lender name',
        max_length=200,
        blank=True,
        help_text="Name of the lender (from LOAN_CD.INTR_NAML)"
    )
    amount_guaranteed_this_period = models.DecimalField(
        verbose_name='amount guaranteed this period',
        decimal_places=2,
        max_digits=14,
        help_text="Amount guaranteed for the period covered by the filing "
                  "(from LOAN_CD.LOAN_AMT1)"
    )
    balance_outstanding_to_date = models.DecimalField(
        verbose_name='balance outstanding to date',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding balance for which the guarantor is liable at "
                  "the close of the reporting period (from LOAN_CD.LOAN_AMT2)"
    )
    cumulative_ytd_amount = models.DecimalField(
        verbose_name='cumulative year-to-date amount',
        decimal_places=2,
        max_digits=14,
        help_text="Cumulative amount guaranteed during the calendar year "
                  "covered by the statement (from LOAN_CD.LOAN_AMT3)"
    )
    loan_date = models.DateField(
        verbose_name='loan date',
        null=True,
        help_text="Date of the loan or date the line of credit was established"
                  "(from LOAN_CD.LOAN_DATE1)"
    )
    interest_rate = models.CharField(
        verbose_name='interest rate',
        max_length=30,
        blank=True,
        help_text='Interest rate of the loan. This is sometimes expressed as a '
                  'decimal (e.g., 0.10) and other times as a percent (e.g., '
                  '10.0% (from LOAN_CD.LOAN_RATE)'
    )
    transaction_id = models.CharField(
        verbose_name='transaction id',
        max_length=20,
        help_text='Identifies a unique transaction across versions of the a '
                  'given Form 460 filing (from LOAN_CD.TRAN_ID)',
    )
    memo_reference_number = models.CharField(
        verbose_name='memo reference number',
        max_length=20,
        blank=True,
        help_text="A value assigned by the filer which refers to the item's"
                  "footnote in the TEXT_MEMO_CD table (from LOAN_CD."
                  "MEMO_REFNO)",
    )
    reported_on_b1 = models.BooleanField(
        verbose_name='reported on B1',
        default=False,
        help_text='Indicates if the item was actually reported on Part 1 of '
                  'Schedule B. Until 2001, campaign filers were required to '
                  'report guarantors of loans or lines of credit on Part 1 of '
                  'Schedule B.'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleB2Item(Form460ScheduleB2ItemBase):
    """
    Guarantees of loans and lines of credit received by campaign filers.

    These transactions are itemized on Schedule B, Part 2, of the most recent
    version to each Form 460 filing. For guarantees itemized on
    any version of any Form 460 filing, see Form460ScheduleB2ItemVersion.

    Derived from LOAN_CD records where FORM_TYPE is 'B2'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_b2_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the loan '
                  'was reported (from LOAN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B2 item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleB2ItemVersion(Form460ScheduleB2ItemBase):
    """
    Every version of each guarantee of a loan/line of credit to a campaign filer.

    For guaratees itemized on Schedule B, Part 1, of the most recent
    version of each Form 460 filing, see Form460ScheduleB2Item.

    Derived from LOAN_CD records where FORM_TYPE is 'B2'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_b2_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the outstanding loan',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B2 item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )


class Form460ScheduleB2ItemBaseOld(CampaignLoanReceivedItemBase):
    """
    Abstract base model for Schedule B, Part 2, items from Form 460 circa 2001.

    Until Form 460 was modified in 2001, campaign filers were required to report
    repayments made on loans received, loans forgiven, and loans repaid by a
    third party on Part 2 of Schedule B.
    """
    date_repaid_or_forgiven = models.DateField(
        verbose_name='date paid or forgiven',
        null=True,
        help_text="Date when the loan repayment or forgiveness occurred (from "
                  "LOAN_CD.LOAN_DATE2)"
    )
    date_of_original_loan = models.DateField(
        verbose_name='date of original loan',
        null=True,
        help_text="Date the loan was orginally made (from LOAN_CD.LOAN_DATE1)"
    )
    REPAYMENT_TYPE_CHOICES = (
        ('B2F', 'Forgiven'),
        ('B2R', 'Repay'),
        ('B2T', 'Third party payment'),
    )
    repayment_type = models.CharField(
        verbose_name='repayment type',
        max_length=3,
        choices=REPAYMENT_TYPE_CHOICES,
        help_text='Indicates whether the item is a loan repayment by the '
                  'campaign filer, a repayment by a third-party or a loan '
                  'forgiveness by the lender (from LOAN_CD.LOAN_TYPE)',
    )
    amount_repaid_or_forgiven = models.DecimalField(
        verbose_name='amount repaid or forgiven',
        decimal_places=2,
        max_digits=14,
        help_text="Amount paid back or forgiven during the period covered by "
                  "the filing (from LOAN_CD.LOAN_AMT1)"
    )
    outstanding_principle = models.DecimalField(
        verbose_name='outstanding principle',
        decimal_places=2,
        max_digits=14,
        help_text="Outstanding principle of the loan at the end of the period "
                  "covered by the filing (from LOAN_CD.LOAN_AMT2)"
    )
    interest_paid = models.DecimalField(
        verbose_name='interest paid',
        decimal_places=2,
        max_digits=14,
        help_text="Amount of interest paid on the loan during the period "
                  "covered by the campaign filing (from LOAN_CD.LOAN_AMT3)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleB2ItemOld(Form460ScheduleB2ItemBaseOld):
    """
    Repayments on loans/forgiven loans received by campaign filers circa 2001.

    These transactions are itemized on Schedule B, Part 2, of the most recent
    version to each Form 460 filing in the pre-2001 format. For loan repayments
    and forgiven loans on any version of any Form 460 filing in the pre-2001
    format, see Form460ScheduleB2ItemVersionOld.

    Derived from LOAN_CD records where LOAN_TYPE is not blank or LOAN_DATE1 is
    before December 22, 2000.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_b2_items_old',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the loan '
                  'transaction was reported (from LOAN_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B2 old item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleB2ItemVersionOld(Form460ScheduleB2ItemBaseOld):
    """
    Every version of each repayment/forgiveness of a loan to a campaign filer circa 2001.

    For loan repayments and forgiven loans on any version of any Form 460 filing
    in the pre-2001 format, see Form460ScheduleB2ItemOld.

    Derived from LOAN_CD records where LOAN_TYPE is not blank or LOAN_DATE1 is
    before December 22, 2000.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_b2_items_old',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the loan transaction'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule B2 old item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom manager for loading raw data in to "filings" models.
"""
from __future__ import unicode_literals
import itertools

# Django tricks
from django.db.models import Q
from django.db import connection

# Managers
from calaccess_processed.managers import BulkLoadSQLManager

# Logging
import logging
logger = logging.getLogger(__name__)


class FilingsManager(BulkLoadSQLManager):
    """
    Utilities for more quickly loading bulk data.
    """
    app_name = "calaccess_processed_filings"

    def get_sql(self):
        """
        Return string of raw sql for loading the model.
        """
        fp = open(self.sql_path, 'r')
        sql = fp.read()
        fp.close()
        return sql

    @property
    def sql_path(self):
        """
        Return the path to the .sql file with the model's loading query.
        """
        file_name = 'load_%s_model' % self.model._meta.model_name
        return self.get_sql_path(file_name)

    def load(self):
        """
        Load the model by executing its corresponding raw SQL query.

        Temporarily drops any constraints or indexes on the model.
        """
        # Drop constraints and indexes to speed loading
        self.get_queryset().drop_constraints()
        self.get_queryset().drop_indexes()

        # Run the actual loader SQL
        with connection.cursor() as c:
            c.execute(self.get_sql())

        # Restore the constraints and index that were dropped
        self.get_queryset().restore_constraints()
        self.get_queryset().restore_indexes()


class Form501FilingManager(FilingsManager):
    """
    A custom manager for Form 501 filings.
    """
    def without_candidacy(self):
        """
        Returns Form 501 filings that do not have an OCD Candidacy yet.
        """
        from calaccess_processed_elections.proxies import OCDCandidacyProxy

        matched_qs = OCDCandidacyProxy.objects.matched_form501_ids()
        matched_list = [i for i in itertools.chain.from_iterable(matched_qs)]
        return self.get_queryset().exclude(
            Q(filing_id__in=matched_list) | Q(office__icontains='RETIREMENT')
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load OCD elections models with data extracted and scraped from CAL-ACCESS.
"""
import os
from django.apps import apps
from django.conf import settings
from django.utils.timezone import now
from django.core.management import call_command
from . import LoadOCDElectionsBase


class Command(LoadOCDElectionsBase):
    """
    Load OCD elections models with data extracted and scraped from CAL-ACCESS.
    """
    help = 'Load OCD elections models with data extracted and scraped from CAL-ACCESS'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Get the tracking model instance for this version
        self.processed_version = self.get_or_create_processed_version()[0]

        # create subdirectory in processed_data_dir, if missing
        filings_data_path = os.path.join(self.processed_data_dir, 'relational')
        os.path.isdir(filings_data_path) or os.makedirs(filings_data_path)

        # Start off loading all the data
        self.load()

        # archive if django project setting enabled
        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            # then archive
            if self.verbosity > 2:
                self.log(' Archiving OCD processed data files.')
            self.archive()

        # Wrap it up
        self.success('Done!')

    def load(self):
        """
        Load all of the processed models.
        """
        # Set options for commands
        options = dict(verbosity=self.verbosity, no_color=self.no_color)

        #
        # Load parties
        #

        call_command('loadocdparties', **options)
        self.duration()

        #
        # Load elections
        #

        call_command('loadocdballotmeasureelections', **options)
        self.duration()

        call_command('loadocdcandidateelections', **options)
        self.duration()

        #
        # Load contests and candidates
        #

        call_command('loadocdcandidatecontests', **options)
        self.duration()

        call_command('loadocdballotmeasurecontests', **options)
        self.duration()

        call_command('loadocdretentioncontests', **options)
        self.duration()

        call_command('loadocdcandidaciesfrom501s', **options)
        self.duration()

        call_command('loadocdincumbentofficeholders', **options)
        self.duration()

        #
        # Merge duplicates
        #

        call_command('mergeocdpersonsbyfilerid', **options)
        self.duration()

        call_command('mergeocdpersonsbycontestandname', **options)
        self.duration()

    def archive(self):
        """
        Save a csv file for each loaded OCD model.
        """
        models_to_archive = apps.get_app_config("calaccess_processed_elections").get_ocd_models_list()

        for m in models_to_archive:
            obj, created = self.processed_version.files.get_or_create(file_name=m._meta.object_name)
            obj.process_start_datetime = now()
            obj.save()

            call_command('archivecalaccesselectionsfile', m._meta.object_name)
            obj.refresh_from_db()
            obj.process_finish_datetime = now()
            obj.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy model for augmenting ScrapedIncumbent model with methods useful for processing.
"""
from __future__ import unicode_literals
from .base import ScrapedNameMixin
from calaccess_scraped.models import Incumbent


class ScrapedIncumbentProxy(Incumbent, ScrapedNameMixin):
    """
    A proxy for the calaccess_scraped Incumbent model.
    """
    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True
        ordering = ['-session']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for generating flatfiles that combine multiple table into a simplified file.
"""
from __future__ import unicode_literals
from opencivicdata.elections.models import RetentionContest
from calaccess_processed.proxies import OCDProxyModelMixin
from calaccess_processed_flatfiles.managers import OCDFlatRetentionContestManager


class OCDFlatRetentionContestProxy(RetentionContest, OCDProxyModelMixin):
    """
    Every recall measure.
    """
    objects = OCDFlatRetentionContestManager()

    copy_to_fields = (
        ('name',),
        ('office_holder',
         'Name of the office holder.'),
        ('office',
         'Office held.'),
        ('election_name',
         'Name of the election in which the ballot measure is decided.'),
        ('election_date',
         'Date of the election in which the ballot measure is decided.'),
        ('description',),
        ('ocd_contest_id',),
        ('ocd_person_id',
         'Reference to the Person that is the office holder.'),
        ('ocd_post_id',
         'Reference to the Post that is the office.'),
        ('ocd_membership_id',
         RetentionContest._meta.get_field('membership').help_text),
        ('ocd_election_id',
         RetentionContest._meta.get_field('election').help_text),
        ('created_at',),
        ('updated_at',),
        ('calaccess_measure_id',
         'Identifier assigned to the ballot measure by CAL-ACCESS.'),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_flatfiles"
        proxy = True
        verbose_name_plural = 'recall measures'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy model for augmenting ScrapedCandidateElection model with methods useful for processing.
"""
from __future__ import unicode_literals, absolute_import
import re
from datetime import date
from django.utils import timezone
from calaccess_processed_elections import get_expected_election_date, special_elections

# Models
from calaccess_scraped.models import CandidateElection
from .base import ScrapedElectionProxyMixin
from .incumbentelections import ScrapedIncumbentElectionProxy
from calaccess_processed_elections.proxies import OCDElectionProxy


class ScrapedCandidateElectionProxy(ScrapedElectionProxyMixin, CandidateElection):
    """
    A proxy for the CandidateElection model in calaccess_scraped.
    """
    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def get_ocd_election(self):
        """
        Returns an OCD Election object for this record, if it exists.
        """
        # First, try getting the record via election's scraped_id
        try:
            return OCDElectionProxy.objects.get(
                identifiers__scheme='calaccess_election_id',
                identifiers__identifier=self.scraped_id,
            )
        except OCDElectionProxy.DoesNotExist:
            # If that doesn't exist, try getting it by date
            if self.date:
                return OCDElectionProxy.objects.get(date=self.date)
            else:
                # If that fails raise the DoesNotExist error
                raise

    @property
    def election_type(self):
        """
        Return the scraped incumbent election's type.

        (e.g., "GENERAL", "PRIMARY", "SPECIAL ELECTION", "SPECIAL RUNOFF")
        """
        return self.parsed_name['type']

    @property
    def is_recall(self):
        """
        This election is held to recall or retain one or more office holders.
        """
        return 'RECALL' in self.election_type

    @property
    def is_regular(self):
        """
        This is a regularly scheduled primary or general election.
        """
        return self.election_type == 'PRIMARY' or self.election_type == 'GENERAL'

    @property
    def is_special(self):
        """
        This is a special election or runoff outside of the regular election calendar.
        """
        return 'SPECIAL' in self.election_type

    @property
    def date(self):
        """
        Use a scraped candidate election name to look up the election date.

        Return a timezone aware date object, if found, else None.
        """
        # If this is the 2008, we have a hacked out edge case solution
        if self.name == '2008 PRIMARY':
            return date(2008, 6, 3)

        try:
            # Check if the date is in our hardcoded list of special election
            dt = special_elections.names_to_dates_dict[self.name]
            return timezone.datetime.strptime(dt, '%Y-%m-%d').date()
        except KeyError:
            pass

        # If not check the alternative list kept by the scraped IncumbentElection model
        try:
            incumbent_election = ScrapedIncumbentElectionProxy.objects.get(
                date__year=self.parsed_name['year'],
                name__icontains=self.parsed_name['type'],
            )
            return incumbent_election.date
        except (
            ScrapedIncumbentElectionProxy.DoesNotExist,
            ScrapedIncumbentElectionProxy.MultipleObjectsReturned,
        ):
            pass

        # If that doesn't work either, try parsing the election date from the name
        try:
            return get_expected_election_date(self.date.year, self.election_type)
        except ValueError:
            # If that fails, raise exception
            raise Exception(
                'Unknown date for %s. Check http://www.sos.ca.gov/elections/ '
                'and try adding missing date to '
                'calaccess_processed/special_elections.py' % self
            )

    @property
    def parsed_name(self):
        """
        Parse a scraped candidate election name into its constituent parts.

        Parts include:
        * Four-digit year (int)
        * Type (str), e.g., "GENERAL", "PRIMARY", "SPECIAL ELECTION", "SPECIAL RUNOFF"
        * Office (optional str)
        * District (optional int)

        Returns a dict with year, type, office and district as keys.
        """
        # Parse out the data
        pattern = r'^(?P<year>\d{4}) (?P<type>\b(?:[A-Z]| )+)(?: \((?P<office>(?:[A-Z]| )+)(?P<district>\d+)?\))?$' # NOQA
        parsed_name = re.match(pattern, self.name).groupdict()

        # Clean up the contents
        parsed_name['year'] = int(parsed_name['year'])
        parsed_name['type'] = parsed_name['type'].strip()

        # A special carveout for a 2018 election we know is a recall but the site has wrong.
        # http://www.sos.ca.gov/elections/upcoming-elections/2018-recall-sd29/
        if self.name == '2018 SPECIAL ELECTION (STATE SENATE 29)':
            parsed_name['type'] = "SPECIAL RECALL"

        if parsed_name['office']:
            parsed_name['office'] = parsed_name['office'].strip()
        if parsed_name['district']:
            parsed_name['district'] = int(parsed_name['district'])

        # Pass it out
        return parsed_name



# Generated by Django 2.0.4 on 2018-05-11 00:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0009_auto_20180427_0011'),
    ]

    operations = [
        migrations.RenameField(
            model_name='form460filing',
            old_name='begin_cash_balance',
            new_name='beginning_cash_balance',
        ),
        migrations.RenameField(
            model_name='form460filingversion',
            old_name='begin_cash_balance',
            new_name='beginning_cash_balance',
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Basic configuration for the application.
"""
from __future__ import unicode_literals
import os
from django.apps import AppConfig


class CalAccessProcessedConfig(AppConfig):
    """
    Application configuration.
    """
    name = 'calaccess_processed'
    verbose_name = "CAL-ACCESS processed data"
    default_auto_field = 'django.db.models.AutoField'
    # Where SQL files are stored in this application
    sql_directory_path = os.path.join(os.path.dirname(__file__), 'sql')

    def get_processed_file_lookup(self):
        """
        Returns a dictionary with the names of processed files mapped to models.
        """
        from django.apps import apps
        d = {}
        filings = apps.get_app_config("calaccess_processed_filings").get_filing_model_lookup()
        elections = apps.get_app_config("calaccess_processed_elections").get_ocd_proxy_lookup()
        flat = apps.get_app_config("calaccess_processed_flatfiles").get_flat_proxy_lookup()
        d.update(filings)
        d.update(elections)
        d.update(flat)
        return d



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for generating flatfiles that combine multiple table into a simplified file.
"""
from __future__ import unicode_literals
from calaccess_processed.proxies import OCDProxyModelMixin
from opencivicdata.elections.models import Candidacy, CandidateContest
from calaccess_processed_flatfiles.managers import OCDFlatCandidacyManager


class OCDFlatCandidacyProxy(Candidacy, OCDProxyModelMixin):
    """
    Every candidate for a public office.
    """
    objects = OCDFlatCandidacyManager()

    copy_to_fields = (
        ('name',),
        ('party_name',
         'Name of the political party that nominated the candidate or would '
         'nominate the candidate (as in the case of a partisan primary).',),
        ('election_name',),
        ('election_date',),
        ('office',
         'Public office for which the candidate is seeking election.',),
        ('is_incumbent',),
        ('special_election', CandidateContest._meta.get_field('previous_term_unexpired').help_text),
        ('created_at',),
        ('updated_at',),
        ('ocd_person_id', Candidacy._meta.get_field('person').help_text),
        ('ocd_candidacy_id',),
        ('ocd_election_id', CandidateContest._meta.get_field('election').help_text),
        ('ocd_post_id', Candidacy._meta.get_field('post').help_text),
        ('ocd_contest_id', Candidacy._meta.get_field('contest').help_text),
        ('ocd_party_id', Candidacy._meta.get_field('party').help_text),
        ('latest_calaccess_filer_id',
         'Most recent filer_id assigned to the person in CAL-ACCESS.',),
        ('calaccess_filer_id_count',
         'Count of filer_ids assigned to the person in CAL-ACCESS.',),
        ('latest_form501_filing_id', "CAL-ACCESS identifier for the candidate's "
                                     "most recent Form 501 filing."),
        ('form501_filing_count', 'Count of Form 501s filed by the candidate.'),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_flatfiles"
        proxy = True
        verbose_name_plural = 'candidates'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Check for mistakes in processed data loaded from CAL-ACCESS.
"""
from django.utils.timezone import now
from calaccess_processed.management.commands import CalAccessCommand
# from calaccess_processed_filings.models import (
#     Form460Filing,
#     Form460FilingVersion,
# )
from calaccess_processed_elections.proxies import (
    OCDElectionProxy,
    OCDMembershipProxy
)
# from calaccess_processed_campaignfinance.proxies import (
#     OCDFilingProxy,
#     OCDFilingActionProxy
# )


class Command(CalAccessCommand):
    """
    Check for mistakes in processed data loaded from CAL-ACCESS.
    """
    help = 'Check for mistakes in processed data loaded from CAL-ACCESS.'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        exec_ok, exec_msg = self.test_regular_executive_contest_counts()
        if exec_ok:
            self.success('  State Executive office contests verified.')
        else:
            self.failure(exec_msg)

        sen_ok, sen_msg = self.test_regular_senate_contest_counts()
        if sen_ok:
            self.success('  State Senate contests verified.')
        else:
            self.failure(sen_msg)

        sen_dists_ok, sen_dists_msg = self.test_regular_senate_contest_districts()
        if sen_dists_ok:
            self.success('  State Senate districts verified.')
        else:
            self.failure(sen_dists_msg)

        asm_ok, asm_msg = self.test_regular_assembly_contest_counts()
        if asm_ok:
            self.success('  State Assembly contests verified.')
        else:
            self.failure(asm_msg)

        mems_ok, mems_msg = self.test_for_duplicate_memberships()
        if mems_ok:
            self.success('  No duplicate office holders')
        else:
            self.failure(mems_msg)

        # form460_filings_ok, mems_msg = self.test_form460_filings_count()
        # if mems_ok:
        #     self.success('  Form 460 filings verified')
        # else:
        #     self.failure(mems_msg)

        # form460_filing_actions_ok, mems_msg = self.test_form460_filing_actions_count()
        # if mems_ok:
        #     self.success('  Form 460 filing actions verified')
        # else:
        #     self.failure(mems_msg)

    def test_regular_assembly_contest_counts(self):
        """
        Confirm equality of actual and expected counts of assembly contests in each election.

        Test only prior elections.

        Return a tuple with boolean value (True means passed) and error message.
        """
        bad_elections = [
            e for e in OCDElectionProxy.objects.filter(date__year__lte=now().year)
            if e.regular_assembly_contest_count_actual != e.regular_assembly_contest_count_expected
        ]
        error_count = len(bad_elections)
        error_list = '\n'.join([
            '- {0} should have {1}, but has {2}'.format(
                e, e.regular_assembly_contest_count_expected, e.regular_assembly_contest_count_actual
            ) for e in bad_elections]
        )
        msg = 'Actual and expected counts of assembly contests not equal in {0} \
elections:\n{1}'.format(error_count, error_list)

        return (error_count == 0, msg)

    def test_regular_executive_contest_counts(self):
        """
        Confirm equality of actual and expected counts of senate contests in each election.

        Test only prior elections.

        Return a tuple with boolean value (True means passed) and error message.
        """
        bad_elections = [
            e for e in OCDElectionProxy.objects.filter(date__year__lte=now().year)
            if e.regular_executive_contest_count_actual != e.regular_executive_contest_count_expected
        ]
        error_count = len(bad_elections)
        error_list = '\n'.join([
            '- {0} should have {1}, but has {2}'.format(
                e, e.regular_executive_contest_count_expected, e.regular_executive_contest_count_actual
            ) for e in bad_elections]
        )
        msg = 'Actual and expected counts of executive contests not equal in {0} \
elections:\n{1}'.format(error_count, error_list)

        return (error_count == 0, msg)

    def test_regular_senate_contest_counts(self):
        """
        Confirm equality of actual and expected counts of senate contests in each election.

        Test only prior elections.

        Return a tuple with boolean value (True means passed) and error message.
        """
        bad_elections = [
            e for e in OCDElectionProxy.objects.filter(date__year__lte=now().year)
            if e.regular_senate_contest_count_actual != e.regular_senate_contest_count_expected
        ]
        error_count = len(bad_elections)
        error_list = '\n'.join([
            '- {0} should have {1}, but has {2}'.format(
                e, e.regular_senate_contest_count_expected, e.regular_senate_contest_count_actual
            ) for e in bad_elections]
        )
        msg = 'Actual and expected counts of senate contests not equal in {0} \
elections:\n{1}'.format(error_count, error_list)

        return (error_count == 0, msg)

    def test_regular_senate_contest_districts(self):
        """
        Confirm that no elections have senate contests in the wrong districts.

        Return a tuple with boolean value (True means passed) and error message.
        """
        bad_elections = {}
        for e in OCDElectionProxy.objects.all():
            bad_senate_contests = e.get_regular_senate_contests_in_wrong_districts()
            if len(bad_senate_contests) > 0:
                bad_elections[e] = bad_senate_contests
        error_list = '\n'.join([
            '- {0} has {1}:\n{2}'.format(
                k, len(v), '\n'.join([' - %s' % c for c in v])
            ) for k, v in bad_elections.items()
        ])
        msg = '{0} elections with State Senate contests in the wrong districts:\n{1}'.format(
            len(bad_elections), error_list
        )

        return (len(bad_elections) == 0, msg)

    def test_for_duplicate_memberships(self):
        """
        Confirm there are no duplicate membership records.

        Return a tuple with boolean value (True means passed) and error message.
        """
        dupes = OCDMembershipProxy.objects.get_duplicates()
        error_list = '\n'.join([
            '- {0} in {1} repeated in {2} rows.'.format(
                i.person, i.post, i.row_count
            ) for i in dupes]
        )
        msg = '{0} duplicated Memberships:\n{1}'.format(dupes.count(), error_list)

        return (not dupes.exists(), msg)

    # def test_form460_filings_count(self):
    #     """
    #     Confirm count of Form460Filing is equal to OCD Filing count.
    #     """
    #     processed_f460_count = Form460Filing.objects.count()
    #     ocd_filings_count = OCDFilingProxy.objects.filter(
    #         classification='Form 460',
    #     ).count()
    #
    #     msg = 'Form460Filing count is {0}, but Filing count is {1}.'.format(
    #         processed_f460_count, ocd_filings_count
    #     )
    #     return(processed_f460_count == ocd_filings_count, msg)
    #
    # def test_form460_filing_actions_count(self):
    #     """
    #     Confirm count of Form460FilingVersion is equal to OCD FilingAction count.
    #     """
    #     processed_f460_version_count = Form460FilingVersion.objects.count()
    #     ocd_filing_actions_count = OCDFilingActionProxy.objects.filter(
    #         filing__classification='Form 460',
    #     ).count()
    #
    #     msg = 'Form460FilingVersion count is {0}, but FilingAction count is {1}.'.format(
    #         processed_f460_version_count, ocd_filing_actions_count
    #     )
    #     return(processed_f460_version_count == ocd_filing_actions_count, msg)



import os
import sys
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
SECRET_KEY = 'r269$heh9at2cot+5l$*$4&xzwsfbbg0&&^prr+e&oh)_4-+ga'
DEBUG = True
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_ROOT = os.path.join(BASE_DIR, ".static")
STATIC_URL = '/static/'
ALLOWED_HOSTS = [
    'localhost',
    '127.0.0.1',
]
ROOT_URLCONF = 'project.urls'
WSGI_APPLICATION = 'project.wsgi.application'

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'calaccess_raw',
    'calaccess_scraped',
    'calaccess_processed',
    'calaccess_processed_elections',
    'calaccess_processed_filings',
    'calaccess_processed_flatfiles',
    # 'calaccess_processed_campaignfinance',
    # 'toolbox',
    'opencivicdata.core.apps.BaseConfig',
    'opencivicdata.elections.apps.BaseConfig',
    # 'opencivicdata.campaign_finance.apps.BaseConfig',
)

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'calaccess_processed',
        'USER': 'postgres',
        'PASSWORD': '',
        'HOST': 'localhost',
        'PORT': '5432'
    }
}

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = '/static/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
        },
    },
    'loggers': {
        'calaccess_raw.management': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_scraped.management': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_processed': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_processed_campaignfinance': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_processed_elections': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_processed_filings': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
        'calaccess_processed_flatfiles': {
            'handlers': ['console',],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}

IA_STORAGE_ACCESS_KEY = os.getenv('IA_STORAGE_ACCESS_KEY')
IA_STORAGE_SECRET_KEY = os.getenv('IA_STORAGE_SECRET_KEY')
IA_STORAGE_COLLECTION = 'test_collection'
IA_STORAGE_CONTRIBUTOR = 'palewire'
IA_STORAGE_CREATOR = "palewire"
IA_STORAGE_PUBLISHER = 'california-civic-data-coalition/django-calaccess-processed-data'
IA_STORAGE_MEDIATYPE = "data"
IA_STORAGE_SUBJECT = ['test']

try:
    from .settings_local import *
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy model for augmenting ScrapedIncumbentElection model with methods useful for processing.
"""
from __future__ import unicode_literals

# Models
from .base import ScrapedElectionProxyMixin
from calaccess_scraped.models import IncumbentElection
from calaccess_processed_elections.proxies import OCDElectionProxy

# Managers
from calaccess_processed_elections.managers import ScrapedIncumbentElectionManager


class ScrapedIncumbentElectionProxy(ScrapedElectionProxyMixin, IncumbentElection):
    """
    A proxy for the IncumbentElection model in calaccess_scraped.
    """
    objects = ScrapedIncumbentElectionManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def get_ocd_election(self):
        """
        Returns an OCD Election object for this record, if it exists.
        """
        try:
            ocd_election = OCDElectionProxy.objects.get(
                name=self.ocd_name,
                date=self.date,
            )
        except OCDElectionProxy.DoesNotExist:
            # If that doesn't exist, try getting it by date
            try:
                ocd_election = OCDElectionProxy.objects.get(date=self.date)
            except (
                OCDElectionProxy.DoesNotExist,
                OCDElectionProxy.MultipleObjectsReturned
            ):
                raise

        return ocd_election

    @property
    def election_type(self):
        """
        Return the scraped incumbent election's type.

        (e.g., "GENERAL", "PRIMARY", "SPECIAL ELECTION", "SPECIAL RUNOFF")
        """
        if self.name == 'SPECIAL ELECTION':
            election_type = self.name
        else:
            election_type = self.name.replace('ELECTION', '').strip()

        return election_type



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Schedule 496, the Late Independent Expenditure Report.

More about the filing: https://calaccess.californiacivicdata.org/documentation/calaccess-forms/f496/
"""
from __future__ import unicode_literals

# Models
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignFinanceFilingBase


class Form496Filing(CampaignFinanceFilingBase):
    """
    The most recent version of each Schedule 496 filing by campaign filers.

    Form 496 is filed by committees that make independent expenditures whose combined total is $1,000
    or more to support or oppose a single candidate for office or a single ballot measure.
    Form 496 should be filed within 24-hours of making the expenditure during the 90 days
    immediately preceding the election.

    Includes information from the cover sheet of the most recent amendment to
    each filing.
    """
    filing_id = models.IntegerField(
        verbose_name='filing id',
        primary_key=True,
        null=False,
        help_text='Unique identification number for the Schedule 496 filing ('
                  'from CVR_CAMPAIGN_DISCLOSURE_CD.FILING_ID)',
    )
    amendment_count = models.IntegerField(
        verbose_name='Count amendments',
        null=False,
        help_text='Number of amendments to the Schedule 496 filing (from '
                  'maximum value of CVR_CAMPAIGN_DISCLOSURE_CD.AMEND_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        index_together = ((
            'filing_id',
            'amendment_count',
        ),)
        verbose_name = "Form 496 (Late Independent Expenditure) filing"

    def __str__(self):
        return str(self.filing_id)


class Form496FilingVersion(CampaignFinanceFilingBase):
    """
    Every version of each Schedule 496 filing by a campaign filer.

    Includes information found on the cover sheet of each version of each
    Schedule 496 filing.
    """
    filing = models.ForeignKey(
        'Form496Filing',
        related_name='versions',
        db_constraint=False,
        null=True,
        on_delete=models.SET_NULL,
        help_text='Unique identification number for the Schedule 496 filing ('
                  'from S496_CD.FILING_ID)',
    )
    amend_id = models.IntegerField(
        null=False,
        help_text='Identifies the version of the Schedule 496 filing, with 0 '
                  'representing the initial filing (from CVR_CAMPAIGN_'
                  'DISCLOSURE_CD.AMEND_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'amend_id',
        ),)
        index_together = ((
            'filing',
            'amend_id',
        ),)
        verbose_name = "Form 496 (Late Independent Expenditure) filing version"

    def __str__(self):
        return '%s-%s' % (self.filing, self.amend_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 461).
"""
from .filing import Form461Filing, Form461FilingVersion
from .part5 import (
    Form461Part5Item,
    Form461Part5ItemVersion
)

__all__ = (
    "Form461Filing",
    "Form461FilingVersion",
    "Form461Part5Item",
    "Form461Part5ItemVersion"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load the OCD Election model from the scraped PropositionElection model.
"""
from __future__ import unicode_literals
from calaccess_processed_elections.proxies import ScrapedPropositionElectionProxy
from calaccess_processed_elections.management.commands import LoadOCDElectionsBase


class Command(LoadOCDElectionsBase):
    """
    Load the OCD Election model from the scraped PropositionElection model.
    """
    help = 'Load the OCD Election model from the scraped PropositionElection model'

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)
        self.header("Loading Elections from scraped propositions")
        self.load_from_proxy(ScrapedPropositionElectionProxy)
        self.success("Done!")



# Generated by Django 2.0.4 on 2018-04-24 22:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Form496Filing',
            fields=[
                ('date_filed', models.DateField(db_index=True, help_text='Date this report was filed, according to the filer (from CVR_CAMPAIGN_DISCLOSURE.RPT_DATE)', verbose_name='date filed')),
                ('filer_id', models.IntegerField(db_index=True, help_text='Numeric filer identification number (from FILER_XREF.FILER_ID)', verbose_name='filer id')),
                ('filer_lastname', models.CharField(help_text='Last name of filer (from CVR_CAMPAIGN_DISCLOSURE.FILER_NAML)', max_length=200, verbose_name='filer last name')),
                ('filer_firstname', models.CharField(blank=True, help_text='First name of the filer (from CVR_CAMPAIGN_DISCLOSURE.FILER_NAMF)', max_length=45, verbose_name='filer first name')),
                ('election_date', models.DateField(db_index=True, help_text='Date of the election in which the filer is participating (from CVR_CAMPAIGN_DISCLOSURE.ELECT_DATE)', null=True, verbose_name='election date')),
                ('filing_id', models.IntegerField(help_text='Unique identification number for the Schedule 496 filing (from CVR_CAMPAIGN_DISCLOSURE_CD.FILING_ID)', primary_key=True, serialize=False, verbose_name='filing id')),
                ('amendment_count', models.IntegerField(help_text='Number of amendments to the Schedule 496 filing (from maximum value of CVR_CAMPAIGN_DISCLOSURE_CD.AMEND_ID)', verbose_name='Count amendments')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) filing',
            },
        ),
        migrations.CreateModel(
            name='Form496FilingVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_filed', models.DateField(db_index=True, help_text='Date this report was filed, according to the filer (from CVR_CAMPAIGN_DISCLOSURE.RPT_DATE)', verbose_name='date filed')),
                ('filer_id', models.IntegerField(db_index=True, help_text='Numeric filer identification number (from FILER_XREF.FILER_ID)', verbose_name='filer id')),
                ('filer_lastname', models.CharField(help_text='Last name of filer (from CVR_CAMPAIGN_DISCLOSURE.FILER_NAML)', max_length=200, verbose_name='filer last name')),
                ('filer_firstname', models.CharField(blank=True, help_text='First name of the filer (from CVR_CAMPAIGN_DISCLOSURE.FILER_NAMF)', max_length=45, verbose_name='filer first name')),
                ('election_date', models.DateField(db_index=True, help_text='Date of the election in which the filer is participating (from CVR_CAMPAIGN_DISCLOSURE.ELECT_DATE)', null=True, verbose_name='election date')),
                ('amend_id', models.IntegerField(help_text='Identifies the version of the Schedule 496 filing, with 0 representing the initial filing (from CVR_CAMPAIGN_DISCLOSURE_CD.AMEND_ID)')),
                ('filing', models.ForeignKey(db_constraint=False, help_text='Unique identification number for the Schedule 496 filing (from S496_CD.FILING_ID)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='versions', to='calaccess_processed_filings.Form496Filing')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) filing version',
            },
        ),
        migrations.AlterIndexTogether(
            name='form496filing',
            index_together={('filing_id', 'amendment_count')},
        ),
        migrations.AlterUniqueTogether(
            name='form496filingversion',
            unique_together={('filing', 'amend_id')},
        ),
        migrations.AlterIndexTogether(
            name='form496filingversion',
            index_together={('filing', 'amend_id')},
        ),
    ]



# Generated by Django 2.0.4 on 2018-04-26 23:58

from django.db import migrations, models
from datetime import date


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0007_auto_20180426_2354'),
    ]

    operations = [
        migrations.AddField(
            model_name='form461filing',
            name='from_date',
            field=models.DateField(db_index=True, default=date.today(), help_text='The first date of the filing period covered by the statement (from CVR_CAMPAIGN_DISCLOSURE.FROM_DATE)', verbose_name='from date'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='form461filing',
            name='statement_type',
            field=models.CharField(default='', help_text='Type of statement, e.g., "Quarterly", "Semi-Annual", Pre-Election (from CVR_CAMPAIGN_DISCLOSURE.STMT_TYPE)', max_length=50, verbose_name='statement type'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='form461filing',
            name='thru_date',
            field=models.DateField(db_index=True, default=date.today(), help_text='The last date of the filing period covered by the statement (from CVR_CAMPAIGN_DISCLOSURE.THRU_DATE)', verbose_name='thru date'),
            preserve_default=False,
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel
from calaccess_processed_filings.models.campaign import CampaignContributionBase


#
# Summaries
#

class Form460ScheduleASummaryBase(FilingBaseModel):
    """
    Abstract base model with summary data from Schedule A attachments to Form 460 filings.

    Includes totals for itemized versus unitemized contributions.
    """
    itemized_contributions = models.FloatField(
        verbose_name='itemized contributions',
        null=True,
        help_text="Amount received this period - contributions of $100 or more. (Include all Schedule A subtotals.)",
    )
    unitemized_contributions = models.FloatField(
        verbose_name='unitemized contributions',
        null=True,
        help_text="Amount received this period - unitemized contributions of less than $100",
    )
    total_contributions = models.FloatField(
        verbose_name='total contributions',
        null=True,
        help_text="Total monetary contributions received this period. (Add Lines 1 and 2. Enter here and on summary \
Page, Column A, Line 1.)",
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleASummary(Form460ScheduleASummaryBase):
    """
    The summary data included with the most recent version a Schedule A attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized contributions.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_a_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the summary was reported',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule A summary"
        verbose_name_plural = "Form 460 (Campaign Disclosure) Schedule A summaries"

    def __str__(self):
        return '%s' % (self.filing)


class Form460ScheduleASummaryVersion(Form460ScheduleASummaryBase):
    """
    The summary data included with each version of a Schedule A attachment to a Form 460 filing.

    Includes totals for itemized versus unitemized contributions.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_a_summaries',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the summary'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        verbose_name = "Form 460 (Campaign Disclosure) Schedule A summary version"

    def __str__(self):
        return '%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
        )


#
# Items
#

class Form460ScheduleAItemBase(CampaignContributionBase):
    """
    Abstract base model for items reported on Schedule A of Form 460 filings.

    On Schedule A, campaign filers are required to itemize monetary
    contributions received during the period covered by the filing.
    """
    amount = models.DecimalField(
        verbose_name='amount',
        decimal_places=2,
        max_digits=14,
        help_text="Amount received from the contributor in the period covered "
                  "by the filing (from RCPT_CD.AMOUNT)"
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True


class Form460ScheduleAItem(Form460ScheduleAItemBase):
    """
    Monetary contributions received by campaign filers.

    These transactions are itemized on Schedule A of the most recent version
    of each Form 460 filing. For monetary contributions itemized on any version
    of any Form 460 filing, see Form460ScheduleAItemVersion.

    Also includes contributions transferred to special election commitees,
    which were itemized on Schedule A-1 until around 2001.

    Derived from RCPT_CD records where FORM_TYPE is 'A' or 'A-1'.
    """
    filing = models.ForeignKey(
        'Form460Filing',
        related_name='schedule_a_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the Form 460 on which the monetary'
                  ' contribution was reported (from RCPT_CD.FILING_ID)',
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule A item"

    def __str__(self):
        return '%s-%s' % (self.filing, self.line_item)


class Form460ScheduleAItemVersion(Form460ScheduleAItemBase):
    """
    Every version of each monetary contribution received by a campaign filer.

    For monetary contributions itemized on Schedule A of the most recent
    version of each Form 460 filing, see Form460ScheduleAItem.

    Derived from RCPT_CD records where FORM_TYPE is 'A' or 'A-1'.
    """
    filing_version = models.ForeignKey(
        'Form460FilingVersion',
        related_name='schedule_a_items',
        null=True,
        on_delete=models.SET_NULL,
        help_text='Foreign key referring to the version of the Form 460 that '
                  'includes the received contribution'
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        unique_together = ((
            'filing_version',
            'line_item',
        ),)
        index_together = ((
            'filing_version',
            'line_item',
        ),)
        verbose_name = "Form 460 (Campaign Disclosure) Schedule A item version"

    def __str__(self):
        return '%s-%s-%s' % (
            self.filing_version.filing_id,
            self.filing_version.amend_id,
            self.line_item
        )



# Generated by Django 3.2.4 on 2021-06-19 11:39

from django.db import migrations
import ia_storage.fields
import ia_storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed', '0015_auto_20180416_2200'),
    ]

    operations = [
        migrations.AlterField(
            model_name='processeddatafile',
            name='file_archive',
            field=ia_storage.fields.InternetArchiveFileField(blank=True, help_text='An archive of the processed file', max_length=255, storage=ia_storage.storage.InternetArchiveStorage, upload_to='', verbose_name='archive of processed file'),
        ),
        migrations.AlterField(
            model_name='processeddatazip',
            name='zip_archive',
            field=ia_storage.fields.InternetArchiveFileField(help_text='An archived zip of processed files', max_length=255, storage=ia_storage.storage.InternetArchiveStorage, upload_to='', verbose_name='zip archive'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Includes a mapping of special election names to dates.
"""
# This list was compiled from CAL-ACCESS:
# http://cal-access.sos.ca.gov/Campaign/Candidates/

# And also the documents available from the Secretary of State site:
# * http://www.sos.ca.gov/elections/prior-elections/special-elections/
# * http://elections.cdn.sos.ca.gov/special-elections/pdf/special-elections-history.pdf

names_to_dates = (
    ('2021 SPECIAL ELECTION (ASSEMBLY 79)', '2020-04-15'),
    ('2021 SPECIAL ELECTION (STATE SENATE 30)', '2020-04-15'),
    ('2020 SPECIAL RUNOFF (STATE SENATE 28)', '2020-05-12'),
    ('2020 SPECIAL ELECTION (STATE SENATE 28)', '2020-03-03'),
    ('2019 SPECIAL RUNOFF (ASSEMBLY 01)', '2019-11-05'),
    ('2019 SPECIAL ELECTION (ASSEMBLY 01)', '2019-08-27'),
    ('2019 SPECIAL RUNOFF (STATE SENATE 01)', '2019-06-04'),
    ('2019 SPECIAL ELECTION (STATE SENATE 01)', '2019-03-26'),
    ('2019 SPECIAL RUNOFF (STATE SENATE 33)', '2019-06-04'),
    ('2019 SPECIAL ELECTION (STATE SENATE 33)', '2019-03-26'),
    ('2018 SPECIAL RUNOFF (STATE SENATE 32)', '2018-08-07'),
    ('2018 SPECIAL RUNOFF (ASSEMBLY 39)', '2018-06-05'),
    ('2018 SPECIAL RUNOFF (ASSEMBLY 45)', '2018-06-05'),
    ('2018 SPECIAL ELECTION (STATE SENATE 32)', '2018-06-05'),
    # http://www.sos.ca.gov/elections/upcoming-elections/2018-recall-sd29/
    ('2018 SPECIAL ELECTION (STATE SENATE 29)', '2018-06-05'),
    ('2018 SPECIAL ELECTION (ASSEMBLY 54)', '2018-04-03'),
    ('2018 SPECIAL ELECTION (ASSEMBLY 39)', '2018-04-03'),
    ('2018 SPECIAL ELECTION (ASSEMBLY 45)', '2018-04-03'),
    ('2017 SPECIAL RUNOFF (ASSEMBLY 51)', '2017-12-5'),
    ('2017 SPECIAL ELECTION (ASSEMBLY 51)', '2017-10-3'),
    ('2016 SPECIAL ELECTION (ASSEMBLY 31)', '2016-4-5'),
    ('2015 SPECIAL RUNOFF (STATE SENATE 07)', '2015-5-19'),
    ('2015 SPECIAL ELECTION (STATE SENATE 07)', '2015-3-17'),
    ('2015 SPECIAL ELECTION (STATE SENATE 21)', '2015-3-17'),
    ('2015 SPECIAL ELECTION (STATE SENATE 37)', '2015-3-17'),
    ('2014 SPECIAL ELECTION (STATE SENATE 35)', '2014-12-9'),
    ('2014 SPECIAL ELECTION (STATE SENATE 23)', '2014-3-25'),
    ('2013 SPECIAL ELECTION (ASSEMBLY 54)', '2013-12-3'),
    ('2013 SPECIAL RUNOFF (ASSEMBLY 45)', '2013-11-19'),
    ('2013 SPECIAL ELECTION (ASSEMBLY 45)', '2013-9-17'),
    ('2013 SPECIAL RUNOFF (ASSEMBLY 52)', '2013-9-24'),
    ('2013 SPECIAL ELECTION (ASSEMBLY 52)', '2013-7-23'),
    ('2013 SPECIAL ELECTION (STATE SENATE 26)', '2013-9-17'),
    ('2013 SPECIAL RUNOFF (STATE SENATE 16)', '2013-7-23'),
    ('2013 SPECIAL ELECTION (STATE SENATE 16)', '2013-5-21'),
    ('2013 SPECIAL ELECTION (ASSEMBLY 80)', '2013-5-21'),
    ('2013 SPECIAL RUNOFF (STATE SENATE 32)', '2013-5-14'),
    ('2013 SPECIAL ELECTION (STATE SENATE 32)', '2013-3-12'),
    ('2013 SPECIAL ELECTION (STATE SENATE 40)', '2013-3-12'),
    ('2013 SPECIAL ELECTION (STATE SENATE 04)', '2013-1-8'),
    ('2012 SPECIAL ELECTION (STATE SENATE 04)', '2012-11-6'),
    ('2011 SPECIAL RUNOFF (ASSEMBLY 04)', '2011-5-3'),
    ('2011 SPECIAL ELECTION (ASSEMBLY 04)', '2011-3-8'),
    ('2011 SPECIAL ELECTION (STATE SENATE 28)', '2011-2-15'),
    ('2011 SPECIAL ELECTION (STATE SENATE 17)', '2011-2-15'),
    ('2011 SPECIAL RUNOFF (STATE SENATE 01)', '2011-1-4'),
    ('2010 SPECIAL ELECTION (STATE SENATE 01)', '2010-11-2'),
    ('2010 SPECIAL RUNOFF (STATE SENATE 15)', '2010-8-17'),
    ('2010 SPECIAL ELECTION (STATE SENATE 15)', '2010-6-22'),
    ('2010 SPECIAL RUNOFF (STATE SENATE 37)', '2010-6-8'),
    ('2010 SPECIAL ELECTION (STATE SENATE 37)', '2010-4-13'),
    ('2010 SPECIAL RUNOFF (ASSEMBLY 43)', '2010-6-8'),
    ('2010 SPECIAL ELECTION (ASSEMBLY 43)', '2010-4-13'),
    ('2010 SPECIAL RUNOFF (ASSEMBLY 72)', '2010-1-12'),
    ('2009 SPECIAL ELECTION (ASSEMBLY 72)', '2009-11-17'),
    ('2009 SPECIAL ELECTION (ASSEMBLY 51)', '2009-9-1'),
    ('2009 SPECIAL RUNOFF (STATE SENATE 26)', '2009-5-19'),
    ('2009 SPECIAL ELECTION (STATE SENATE 26)', '2009-3-24'),
    ('2008 SPECIAL RUNOFF (ASSEMBLY 55)', '2008-2-5'),
    ('2007 SPECIAL ELECTION (ASSEMBLY 55)', '2007-12-11'),
    ('2007 SPECIAL ELECTION (ASSEMBLY 39)', '2007-5-15'),
    ('2006 SPECIAL RUNOFF (STATE SENATE 35)', '2006-6-6'),
    ('2006 SPECIAL ELECTION (STATE SENATE 35)', '2006-4-11'),
    ('2005 SPECIAL ELECTION (ASSEMBLY 53)', '2005-9-13'),
    ('2003 SPECIAL ELECTION (GOVERNOR)', '2003-10-7'),
    ('2001 SPECIAL ELECTION (ASSEMBLY 49)', '2001-5-15'),
    ('2001 SPECIAL RUNOFF (ASSEMBLY 65)', '2001-4-3'),
    ('2001 SPECIAL ELECTION (ASSEMBLY 65)', '2001-2-6'),
    ('2001 SPECIAL ELECTION (STATE SENATE 24)', '2001-3-6'),
)

names_to_dates_dict = dict(t for t in names_to_dates)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the admins from submodules and thread them together.
"""
from calaccess_raw.admin.base import BaseAdmin
from .campaign.form501 import (
    Form501FilingAdmin,
    Form501FilingVersionAdmin,
)
from .campaign.form460 import (
    Form460FilingAdmin,
    Form460FilingVersionAdmin,
    Form460ScheduleAItemAdmin,
    Form460ScheduleAItemVersionAdmin,
    Form460ScheduleCItemAdmin,
    Form460ScheduleCItemVersionAdmin,
    Form460ScheduleB1ItemAdmin,
    Form460ScheduleB1ItemVersionAdmin,
    Form460ScheduleB2ItemAdmin,
    Form460ScheduleB2ItemVersionAdmin,
    Form460ScheduleB2ItemOldAdmin,
    Form460ScheduleB2ItemVersionOldAdmin,
    Form460ScheduleDItemAdmin,
    Form460ScheduleDItemVersionAdmin,
    Form460ScheduleEItemAdmin,
    Form460ScheduleEItemVersionAdmin,
    Form460ScheduleESubItemAdmin,
    Form460ScheduleESubItemVersionAdmin,
    Form460ScheduleFItemAdmin,
    Form460ScheduleFItemVersionAdmin,
    Form460ScheduleGItemAdmin,
    Form460ScheduleGItemVersionAdmin,
    Form460ScheduleHItemAdmin,
    Form460ScheduleHItemVersionAdmin,
    Form460ScheduleH2ItemOldAdmin,
    Form460ScheduleH2ItemVersionOldAdmin,
    Form460ScheduleIItemAdmin,
    Form460ScheduleIItemVersionAdmin,
)
from .campaign.form461 import (
    Form461FilingAdmin,
    Form461FilingVersionAdmin,
    Form461Part5ItemAdmin,
    Form461Part5ItemVersionAdmin
)
from .campaign.form496 import (
    Form496FilingAdmin,
    Form496FilingVersionAdmin,
    Form496Part1ItemAdmin,
    Form496Part1ItemVersionAdmin,
    Form496Part2ItemAdmin,
    Form496Part2ItemVersionAdmin,
    Form496Part3ItemAdmin,
    Form496Part3ItemVersionAdmin
)
from .campaign.form497 import (
    Form497FilingAdmin,
    Form497FilingVersionAdmin,
    Form497Part1ItemAdmin,
    Form497Part1ItemVersionAdmin,
    Form497Part2ItemAdmin,
    Form497Part2ItemVersionAdmin,
)


__all__ = (
    'BaseAdmin',
    'Form501FilingAdmin',
    'Form501FilingVersionAdmin',
    'Form460FilingAdmin',
    'Form460FilingVersionAdmin',
    'Form460ScheduleAItemAdmin',
    'Form460ScheduleAItemVersionAdmin',
    'Form460ScheduleB1ItemAdmin',
    'Form460ScheduleB1ItemVersionAdmin',
    'Form460ScheduleB2ItemAdmin',
    'Form460ScheduleB2ItemVersionAdmin',
    'Form460ScheduleB2ItemOldAdmin',
    'Form460ScheduleB2ItemVersionOldAdmin',
    'Form460ScheduleCItemAdmin',
    'Form460ScheduleCItemVersionAdmin',
    'Form460ScheduleDItemAdmin',
    'Form460ScheduleDItemVersionAdmin',
    'Form460ScheduleEItemAdmin',
    'Form460ScheduleEItemVersionAdmin',
    'Form460ScheduleESubItemAdmin',
    'Form460ScheduleESubItemVersionAdmin',
    'Form460ScheduleFItemAdmin',
    'Form460ScheduleFItemVersionAdmin',
    'Form460ScheduleGItemAdmin',
    'Form460ScheduleGItemVersionAdmin',
    'Form460ScheduleHItemAdmin',
    'Form460ScheduleHItemVersionAdmin',
    'Form460ScheduleH2ItemOldAdmin',
    'Form460ScheduleH2ItemVersionOldAdmin',
    'Form460ScheduleIItemAdmin',
    'Form460ScheduleIItemVersionAdmin',
    "Form461FilingAdmin",
    "Form461FilingVersionAdmin",
    "Form461Part5ItemAdmin",
    "Form461Part5ItemVersionAdmin",
    "Form496FilingAdmin",
    "Form496FilingVersionAdmin",
    "Form496Part1ItemAdmin",
    "Form496Part1ItemVersionAdmin",
    "Form496Part2ItemAdmin",
    "Form496Part2ItemVersionAdmin",
    "Form496Part3ItemAdmin",
    "Form496Part3ItemVersionAdmin",
    'Form497FilingAdmin',
    'Form497FilingVersionAdmin',
    'Form497Part1ItemAdmin',
    'Form497Part1ItemVersionAdmin',
    'Form497Part2ItemAdmin',
    'Form497Part2ItemVersionAdmin',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the managers from submodules and thread them together.
"""
from .divisions import (
    OCDAssemblyDivisionManager,
    OCDSenateDivisionManager,
    OCDCaliforniaDivisionManager
)
from .jurisdictions import OCDJurisdictionManager
from .organizations import (
    OCDOrganizationManager,
    OCDMembershipManager
)
from .people import OCDPersonManager
from .posts import (
    OCDPostManager,
    OCDAssemblyPostManager,
    OCDExecutivePostManager,
    OCDSenatePostManager
)


__all__ = (
    "OCDAssemblyDivisionManager",
    "OCDSenateDivisionManager",
    "OCDCaliforniaDivisionManager",
    "OCDJurisdictionManager",
    "OCDOrganizationManager",
    "OCDMembershipManager",
    "OCDPersonManager",
    "OCDPostManager",
    "OCDAssemblyPostManager",
    "OCDExecutivePostManager",
    "OCDSenatePostManager"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom managers for the OCD CandidateContest models.
"""
from __future__ import unicode_literals
from django.db.models import Q
from django.db import models
from postgres_copy import CopyQuerySet


class OCDCandidateContestQuerySet(CopyQuerySet):
    """
    Custom helpers for the OCD CandidateContest model that limit it to runoffs.
    """
    def set_parents(self):
        """
        Connect and save parent contests for all runoffs.
        """
        for obj in self.runoffs():
            # Carve out for the duplicate 2010 Assembly 43 runoffs until
            # I can figure out what I broke.
            obj.runoff_for_contest = obj.get_parent()
            obj.save()

    def assembly(self):
        """
        Filter to state assembly contests.
        """
        return self.filter(division__subtype2='sldl')

    def executive(self):
        """
        Filter to executive contests.
        """
        return self.filter(
            Q(posts__post__organization__name='California State Executive Branch')
            | Q(posts__post__organization__parent__name='California State Executive Branch')
        )

    def regular(self):
        """
        Filter to "regular" contests.
        """
        return self.filter(previous_term_unexpired=False)

    def runoffs(self):
        """
        Filter down to runoff CandidateContest instances.
        """
        return self.filter(name__contains='RUNOFF')

    def senate(self):
        """
        Filter to state senate contests.
        """
        return self.filter(division__subtype2='sldu')

    def special(self):
        """
        Filter to "special" contests.
        """
        return self.filter(previous_term_unexpired=True)


OCDCandidateContestManager = models.Manager.from_queryset(OCDCandidateContestQuerySet)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom fields.
"""
from django.core.exceptions import FieldDoesNotExist


class CopyToField(object):
    """
    Class to hold meta data about a field included in a CopyTo query.
    """
    def __init__(self, query, name, **kwargs):
        """
        Create a new instance of CopyToField.
        """
        self.query = query
        self.name = name
        self._help_text = kwargs.get('help_text', None)

    @property
    def choices(self):
        """
        Choices of the field.
        """
        return self.field.choices

    @property
    def description(self):
        """
        Description of the field.
        """
        return self.field.description % self.field.__dict__

    @property
    def field(self):
        """
        Django model Field instance.
        """
        return self.resolved_ref.field

    @property
    def help_text(self):
        """
        Help text of the field.
        """
        if self._help_text:
            help_text = self._help_text
        else:
            # return the help_text of the field as defined on the base model, if there
            try:
                help_text = self.query.model._meta.get_field(self.name).help_text
            except FieldDoesNotExist:
                help_text = self.field.help_text

        return help_text

    @property
    def resolved_ref(self):
        """
        Col expression resolved from reference.
        """
        return self.query.resolve_ref(self.name)



# Generated by Django 2.0.4 on 2018-04-26 17:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0004_form496part1item_form496part1itemversion'),
    ]

    operations = [
        migrations.CreateModel(
            name='Form496Part3Item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(help_text='Line number of the filing form where the contribution is itemized (from RCPT_CD.LINE_ITEM)', verbose_name='line item')),
                ('date_received', models.DateField(help_text='Date the contribution was received (from RCPT_CD.RCPT_DATE)', null=True, verbose_name='date received')),
                ('date_received_thru', models.DateField(help_text='End date for late contributions received over a range of days (from RCPT_CD.DATE_THRU)', null=True, verbose_name='date received thru')),
                ('transaction_type', models.CharField(choices=[('F', 'Forgiven Loan'), ('I', 'Intermediary'), ('R', 'Returned (Negative Amount?)'), ('T', 'Third Party Repayment'), ('X', 'Transfer'), ('', 'Unknown'), ('INC16168', 'INC16168')], help_text='Type of transaction (from RCPT_CD.TRAN_TYPE)', max_length=255, verbose_name='transaction type')),
                ('transaction_id', models.CharField(help_text='Identifies a unique transaction across versions of the a given Form 460 filing (from RCPT_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('memo_reference_number', models.CharField(blank=True, help_text='Reference number for the memo attached to the contribution (from RCPT_CD.MEMO_REFNO)', max_length=20, verbose_name='memo reference number')),
                ('contributor_code', models.CharField(blank=True, choices=[('COM', 'Committee'), ('IND', 'Individual'), ('OFF', 'Officer'), ('OTH', 'Other'), ('PTY', 'Political Party'), ('RCP', 'Recipient committee'), ('SCC', 'Small Contributor Committee')], help_text='Code describing the contributor (from RCPT_CD.ENTITY_CD)', max_length=3, verbose_name='contributor code')),
                ('contributor_committee_id', models.CharField(blank=True, help_text="Contributor's filer identification number, if it is a committee (from RCPT_CD.CMTE_ID)", max_length=9, verbose_name='contributor committee id')),
                ('contributor_title', models.CharField(blank=True, help_text='Name title of the contributor (from RCPT_CD.CTRIB_NAMT)', max_length=10, verbose_name='contributor title')),
                ('contributor_lastname', models.CharField(blank=True, help_text='Last name of the contributor or business name (from RCPT_CD.CTRIB_NAML)', max_length=200, verbose_name='contributor lastname')),
                ('contributor_firstname', models.CharField(help_text='First name of the contributor (from RCPT_CD.CTRIB_NAMF)', max_length=45, verbose_name='contributor firstname')),
                ('contributor_name_suffix', models.CharField(blank=True, help_text='Name suffix of the contributor (from RCPT_CD.CTRIB_NAMS)', max_length=10, verbose_name='contributor name suffix')),
                ('contributor_city', models.CharField(blank=True, help_text='City of the contributor (from RCPT_CD.CTRIB_CITY)', max_length=30, verbose_name='contributor city')),
                ('contributor_state', models.CharField(blank=True, help_text='State of the contributor (from RCPT_CD.CTRIB_ST)', max_length=2, verbose_name='contributor state')),
                ('contributor_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the contributor (from RCPT_CD.CTRIB_ZIP4)', max_length=10, verbose_name='contributor zip')),
                ('contributor_employer', models.CharField(blank=True, help_text='Employer of the contributor (from RCPT_CD.CTRIB_EMP)', max_length=200, verbose_name='contributor employer')),
                ('contributor_occupation', models.CharField(blank=True, help_text='Occupation of the contributor (from RCPT_CD.CTRIB_OCC)', max_length=60, verbose_name='contributor occupation')),
                ('contributor_is_self_employed', models.BooleanField(default=False, help_text='Indicates whether or not the contributor is self-employed(from RCPT_CD.CTRIB_SELF)', verbose_name='contributor is self employed')),
                ('intermediary_committee_id', models.CharField(blank=True, help_text="Intermediary's filer identification number, if it is a committee (from RCPT_CD.INTR_CMTEID)", max_length=9, verbose_name='intermediary committee id')),
                ('intermediary_title', models.CharField(blank=True, help_text='Name title of the intermediary (from RCPT_CD.INTR_NAMT)', max_length=10, verbose_name='intermediary title')),
                ('intermediary_lastname', models.CharField(blank=True, help_text='Last name of the intermediary or business name (from RCPT_CD.INTR_NAML)', max_length=200, verbose_name='intermediary lastname')),
                ('intermediary_firstname', models.CharField(help_text='First name of the intermediary (from RCPT_CD.INTR_NAMF)', max_length=45, verbose_name='intermediary firstname')),
                ('intermediary_name_suffix', models.CharField(blank=True, help_text='Name suffix of the intermediary (from RCPT_CD.INTR_NAMS)', max_length=10, verbose_name='intermediary name suffix')),
                ('intermediary_city', models.CharField(blank=True, help_text='City of the intermediary (from RCPT_CD.INTR_CITY)', max_length=30, verbose_name='intermediary city')),
                ('intermediary_state', models.CharField(blank=True, help_text='State of the intermediary (from RCPT_CD.INTR_ST)', max_length=2, verbose_name='intermediary state')),
                ('intermediary_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the intermediary (from RCPT_CD.INTR_ZIP4)', max_length=10, verbose_name='intermediary zip')),
                ('intermediary_employer', models.CharField(blank=True, help_text='Employer of the intermediary (from RCPT_CD.INTR_EMP)', max_length=200, verbose_name='intermediary employer')),
                ('intermediary_occupation', models.CharField(blank=True, help_text='Occupation of the intermediary (from RCPT_CD.INTR_OCC)', max_length=60, verbose_name='intermediary occupation')),
                ('intermediary_is_self_employed', models.BooleanField(default=False, help_text='(from S497_CD.INTR_SELF)', verbose_name='intermediary is self employed')),
                ('cumulative_ytd_amount', models.DecimalField(decimal_places=2, help_text='Cumulative year-to-date amount given by the contributor as of the given Form 460 filing (from RCPT_CD.CUM_YTD)', max_digits=14, null=True)),
                ('cumulative_election_amount', models.DecimalField(decimal_places=2, help_text='For filers subject to contribution limits, cumulative amount given by the contributor during the election cycle as of the given Form 460 filing (from RCPT_CD.CUM_OTH)', max_digits=14, null=True)),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount received from the contributor in the period covered by the filing (from RCPT_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('interest_rate', models.CharField(blank=True, help_text='Interest rate of a loan. This is sometimes expressed as a decimal (e.g., 0.10) and other times as a percent (e.g., 10.0% (from RCPT_CD.INT_RATE)', max_length=30, verbose_name='interest rate')),
                ('filing', models.ForeignKey(help_text='Foreign key referring to the Form 496 on which the monetary contribution was reported (from RCPT_CD.FILING_ID)', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='part3_items', to='calaccess_processed_filings.Form496Filing')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 3 item',
            },
        ),
        migrations.CreateModel(
            name='Form496Part3ItemVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('line_item', models.IntegerField(help_text='Line number of the filing form where the contribution is itemized (from RCPT_CD.LINE_ITEM)', verbose_name='line item')),
                ('date_received', models.DateField(help_text='Date the contribution was received (from RCPT_CD.RCPT_DATE)', null=True, verbose_name='date received')),
                ('date_received_thru', models.DateField(help_text='End date for late contributions received over a range of days (from RCPT_CD.DATE_THRU)', null=True, verbose_name='date received thru')),
                ('transaction_type', models.CharField(choices=[('F', 'Forgiven Loan'), ('I', 'Intermediary'), ('R', 'Returned (Negative Amount?)'), ('T', 'Third Party Repayment'), ('X', 'Transfer'), ('', 'Unknown'), ('INC16168', 'INC16168')], help_text='Type of transaction (from RCPT_CD.TRAN_TYPE)', max_length=255, verbose_name='transaction type')),
                ('transaction_id', models.CharField(help_text='Identifies a unique transaction across versions of the a given Form 460 filing (from RCPT_CD.TRAN_ID)', max_length=20, verbose_name='transaction id')),
                ('memo_reference_number', models.CharField(blank=True, help_text='Reference number for the memo attached to the contribution (from RCPT_CD.MEMO_REFNO)', max_length=20, verbose_name='memo reference number')),
                ('contributor_code', models.CharField(blank=True, choices=[('COM', 'Committee'), ('IND', 'Individual'), ('OFF', 'Officer'), ('OTH', 'Other'), ('PTY', 'Political Party'), ('RCP', 'Recipient committee'), ('SCC', 'Small Contributor Committee')], help_text='Code describing the contributor (from RCPT_CD.ENTITY_CD)', max_length=3, verbose_name='contributor code')),
                ('contributor_committee_id', models.CharField(blank=True, help_text="Contributor's filer identification number, if it is a committee (from RCPT_CD.CMTE_ID)", max_length=9, verbose_name='contributor committee id')),
                ('contributor_title', models.CharField(blank=True, help_text='Name title of the contributor (from RCPT_CD.CTRIB_NAMT)', max_length=10, verbose_name='contributor title')),
                ('contributor_lastname', models.CharField(blank=True, help_text='Last name of the contributor or business name (from RCPT_CD.CTRIB_NAML)', max_length=200, verbose_name='contributor lastname')),
                ('contributor_firstname', models.CharField(help_text='First name of the contributor (from RCPT_CD.CTRIB_NAMF)', max_length=45, verbose_name='contributor firstname')),
                ('contributor_name_suffix', models.CharField(blank=True, help_text='Name suffix of the contributor (from RCPT_CD.CTRIB_NAMS)', max_length=10, verbose_name='contributor name suffix')),
                ('contributor_city', models.CharField(blank=True, help_text='City of the contributor (from RCPT_CD.CTRIB_CITY)', max_length=30, verbose_name='contributor city')),
                ('contributor_state', models.CharField(blank=True, help_text='State of the contributor (from RCPT_CD.CTRIB_ST)', max_length=2, verbose_name='contributor state')),
                ('contributor_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the contributor (from RCPT_CD.CTRIB_ZIP4)', max_length=10, verbose_name='contributor zip')),
                ('contributor_employer', models.CharField(blank=True, help_text='Employer of the contributor (from RCPT_CD.CTRIB_EMP)', max_length=200, verbose_name='contributor employer')),
                ('contributor_occupation', models.CharField(blank=True, help_text='Occupation of the contributor (from RCPT_CD.CTRIB_OCC)', max_length=60, verbose_name='contributor occupation')),
                ('contributor_is_self_employed', models.BooleanField(default=False, help_text='Indicates whether or not the contributor is self-employed(from RCPT_CD.CTRIB_SELF)', verbose_name='contributor is self employed')),
                ('intermediary_committee_id', models.CharField(blank=True, help_text="Intermediary's filer identification number, if it is a committee (from RCPT_CD.INTR_CMTEID)", max_length=9, verbose_name='intermediary committee id')),
                ('intermediary_title', models.CharField(blank=True, help_text='Name title of the intermediary (from RCPT_CD.INTR_NAMT)', max_length=10, verbose_name='intermediary title')),
                ('intermediary_lastname', models.CharField(blank=True, help_text='Last name of the intermediary or business name (from RCPT_CD.INTR_NAML)', max_length=200, verbose_name='intermediary lastname')),
                ('intermediary_firstname', models.CharField(help_text='First name of the intermediary (from RCPT_CD.INTR_NAMF)', max_length=45, verbose_name='intermediary firstname')),
                ('intermediary_name_suffix', models.CharField(blank=True, help_text='Name suffix of the intermediary (from RCPT_CD.INTR_NAMS)', max_length=10, verbose_name='intermediary name suffix')),
                ('intermediary_city', models.CharField(blank=True, help_text='City of the intermediary (from RCPT_CD.INTR_CITY)', max_length=30, verbose_name='intermediary city')),
                ('intermediary_state', models.CharField(blank=True, help_text='State of the intermediary (from RCPT_CD.INTR_ST)', max_length=2, verbose_name='intermediary state')),
                ('intermediary_zip', models.CharField(blank=True, help_text='Zip code (usually zip5, sometimes zip9) of the intermediary (from RCPT_CD.INTR_ZIP4)', max_length=10, verbose_name='intermediary zip')),
                ('intermediary_employer', models.CharField(blank=True, help_text='Employer of the intermediary (from RCPT_CD.INTR_EMP)', max_length=200, verbose_name='intermediary employer')),
                ('intermediary_occupation', models.CharField(blank=True, help_text='Occupation of the intermediary (from RCPT_CD.INTR_OCC)', max_length=60, verbose_name='intermediary occupation')),
                ('intermediary_is_self_employed', models.BooleanField(default=False, help_text='(from S497_CD.INTR_SELF)', verbose_name='intermediary is self employed')),
                ('cumulative_ytd_amount', models.DecimalField(decimal_places=2, help_text='Cumulative year-to-date amount given by the contributor as of the given Form 460 filing (from RCPT_CD.CUM_YTD)', max_digits=14, null=True)),
                ('cumulative_election_amount', models.DecimalField(decimal_places=2, help_text='For filers subject to contribution limits, cumulative amount given by the contributor during the election cycle as of the given Form 460 filing (from RCPT_CD.CUM_OTH)', max_digits=14, null=True)),
                ('amount', models.DecimalField(decimal_places=2, help_text='Amount received from the contributor in the period covered by the filing (from RCPT_CD.AMOUNT)', max_digits=14, verbose_name='amount')),
                ('interest_rate', models.CharField(blank=True, help_text='Interest rate of a loan. This is sometimes expressed as a decimal (e.g., 0.10) and other times as a percent (e.g., 10.0% (from RCPT_CD.INT_RATE)', max_length=30, verbose_name='interest rate')),
                ('filing_version', models.ForeignKey(help_text='Foreign key referring to the version of the Form 496 that includes the received contribution', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='part3_items', to='calaccess_processed_filings.Form496FilingVersion')),
            ],
            options={
                'verbose_name': 'Form 496 (Late Independent Expenditure) Part 3 item version',
            },
        ),
        migrations.AlterUniqueTogether(
            name='form496part3itemversion',
            unique_together={('filing_version', 'line_item')},
        ),
        migrations.AlterIndexTogether(
            name='form496part3itemversion',
            index_together={('filing_version', 'line_item')},
        ),
        migrations.AlterUniqueTogether(
            name='form496part3item',
            unique_together={('filing', 'line_item')},
        ),
    ]



# Generated by Django 2.0.4 on 2018-04-26 17:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_processed_filings', '0005_auto_20180426_1731'),
    ]

    operations = [
        migrations.AddField(
            model_name='form496part1item',
            name='candidate_id',
            field=models.CharField(blank=True, help_text='Identifies the candidate to whom the contribution is connected (from S497_CD.CAND_ID). This can be translated to the filer_id by joining to FILER_XREF_CD.', max_length=9, verbose_name='candidate id'),
        ),
        migrations.AddField(
            model_name='form496part1itemversion',
            name='candidate_id',
            field=models.CharField(blank=True, help_text='Identifies the candidate to whom the contribution is connected (from S497_CD.CAND_ID). This can be translated to the filer_id by joining to FILER_XREF_CD.', max_length=9, verbose_name='candidate id'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD Transaction related managers.
"""
from __future__ import unicode_literals
import logging
from psycopg2 import sql
from .base import CampaignFinanceManager
logger = logging.getLogger(__name__)


class OCDTransactionManager(CampaignFinanceManager):
    """
    Manager with custom methods for OCD Transaction model.
    """
    def execute(self):
        """
        Load OCD Transactions with data from all Form460 related models.
        """
        from calaccess_processed_filings.models import (
            Form460ScheduleAItem,
            Form460ScheduleAItemVersion,
            Form460ScheduleCItem,
            Form460ScheduleCItemVersion
        )

        # ...from Form 460 Schedule A Items...
        # ...for current filings...
        self.insert_contribution_items(
            Form460ScheduleAItem, 'amount', False
        )
        # ...for old filings...
        self.insert_contribution_items(
            Form460ScheduleAItemVersion, 'amount', False
        )

        # ...from Form 460 Schedule C Items...
        # ...for current filings...
        self.insert_contribution_items(
            Form460ScheduleCItem, 'fair_market_value', True
        )
        # ...for old filings...
        self.insert_contribution_items(
            Form460ScheduleCItemVersion, 'fair_market_value', True
        )

    def insert_contribution_items(self, model, amount_field_name, is_in_kind):
        """
        Insert from Form 460 Schedule A Items.

        Insert only current items by default. If current arg is False, insert
        only old items.
        """
        if 'Version' in model._meta.object_name:
            query_name = 'insert_old_transactions_from_form460_contributions'
        else:
            query_name = 'insert_new_transactions_from_form460_contributions'

        source_table = sql.Identifier(model._meta.db_table)
        source_amount_field = model._meta.get_field(amount_field_name)
        source_amount_column = sql.Identifier(source_amount_field.column)

        self.execute_custom_sql(
            "opencivicdata/campaign_finance/transactions/" + query_name,
            params={'is_in_kind': is_in_kind},
            composables={
                'source_table': source_table,
                'source_amount_column': source_amount_column,
            },
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unittests for management commands.
"""
import os
import shutil
import requests_mock

# Time
import time
from email.utils import formatdate
from django.utils.timezone import now

# Django bits
from django.conf import settings
from django.core.management import call_command
from django.test import TestCase, override_settings, TransactionTestCase
from django.core.management.base import CommandError

# Models
from django.db import models
from calaccess_raw.models import RawDataVersion
from calaccess_processed.models import ProcessedDataVersion
from calaccess_scraped.models import Candidate as ScrapedCandidate
from calaccess_scraped.models import Proposition as ScrapedProposition
# from opencivicdata.core.models import Person
from opencivicdata.elections.models import (
    BallotMeasureContest,
    Candidacy,
    CandidateContest
)


class NoProcessedDataTest(TestCase):
    """
    Tests to run with no data loaded.
    """
    def test_no_raw_data(self):
        """
        Confirm process command will not run without data.
        """
        with self.assertRaises(CommandError):
            call_command("processcalaccessdata", verbosity=3)


@override_settings(CALACCESS_DATA_DIR=os.path.join(settings.BASE_DIR, 'test-data'))
@override_settings(MEDIA_ROOT=os.path.join(settings.BASE_DIR, 'test-data', ".media"))
@override_settings(CALACCESS_STORE_ARCHIVE=False)
class ProcessedDataTest(TransactionTestCase):
    """
    Run and test management commands.
    """
    fixtures = [
        'divisions.json',
        'candidate_election.json',
        'candidate.json',
        'incumbent_election.json',
        'incumbent.json',
        'proposition_election.json',
        'proposition.json',
    ]

    @classmethod
    @requests_mock.Mocker()
    def setUp(cls, m):
        """
        Load data for other tests.
        """
        # fake a previous raw data download
        download_dir = os.path.join(settings.CALACCESS_DATA_DIR, 'download')
        os.path.exists(download_dir) or os.mkdir(download_dir)
        zip_path = os.path.join(
            settings.CALACCESS_DATA_DIR,
            'dbwebexport.zip',
        )
        shutil.copy(zip_path, download_dir)
        rdv = RawDataVersion.objects.create(
            release_datetime=now(),
            update_start_datetime=now(),
            download_start_datetime=now(),
            download_finish_datetime=now(),
            expected_size=os.stat(zip_path).st_size,
        )
        # mock an SoS HEAD response
        imf_datetime = formatdate(
            time.mktime(rdv.release_datetime.timetuple()),
            usegmt=True,
        )
        headers = {
            'Content-Length': str(rdv.expected_size),
            'Accept-Ranges': 'bytes',
            'Last-Modified': imf_datetime,
            'Connection': 'keep-alive',
            'Date': imf_datetime,
            'Content-Type': 'application/zip',
            'ETag': '2320c8-30619331-c54f7dc0',
            'Server': 'Apache/2.2.3 (Red Hat)',
        }
        m.register_uri(
            'HEAD',
            'https://campaignfinance.cdn.sos.ca.gov/dbwebexport.zip',
            headers=headers,
        )

        call_command("updatecalaccessrawdata", verbosity=3, noinput=True)
        call_command("processcalaccessdata", verbosity=3)

    def test_everything(self):
        """
        Test the scraped propostions loaded into the database.
        """
        # Confirm count of scraped propositions with a name that doesn't
        # include "RECALL" equals the count of loaded BallotMeasureContest.
        self.assertEqual(
            ScrapedProposition.objects.exclude(name__icontains='RECALL').count(),
            BallotMeasureContest.objects.count(),
        )

#    def test_correction(self):
        """
        Test that we can retrieve a correction directly.
        """
        from calaccess_processed_elections import corrections
        correx = corrections.candidate_party(
            "WINSTON, ALMA MARIE",
            "2014",
            "PRIMARY",
            "GOVERNOR"
        )
        self.assertEqual(correx.name, "REPUBLICAN")

#    def test_correction_assignment_by_proxy(self):
        """
        Test that a correction is properly being applied when parties are retrieved.
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateProxy
        obj = ScrapedCandidateProxy.objects.get(name='WINSTON, ALMA MARIE')
        self.assertEqual(obj.get_party().name, 'REPUBLICAN')

#    def test_processed_version_completed(self):
        """
        Test that the processed version was completed.
        """
        processed_version = ProcessedDataVersion.objects.latest('process_start_datetime')
        self.assertTrue(processed_version.update_completed)

#    def test_processed_file_finished(self):
        """
        Test that each processed file was marked finished.
        """
        processed_version = ProcessedDataVersion.objects.latest('process_start_datetime')
        for df in processed_version.files.all():
            self.assertTrue(df.process_finish_datetime)

#    def test_scraped_candidates(self):
        """
        Test the scraped candidates loaded into the database.
        """
        # Confirm that the count of scraped candidates equals the count loaded
        # into Candidacy with a scraped source
        # minus one, since Jim Fitzgerald didn't really run in the
        # Dem primary in 2008
        self.assertEqual(
            ScrapedCandidate.objects.count() - 1,
            Candidacy.objects.filter(
                sources__url__contains='http://cal-access.sos.ca.gov/Campaign/Candidates/list.aspx?view=certified' # noqa
            ).count(),
        )
        # For each CandidateContest...
        for contest in CandidateContest.objects.all():
            # Confirm there isn't more than one incumbent
            self.assertTrue(
                contest.candidacies.filter(is_incumbent=True).count() <= 1,
                msg="Multiple incumbents in {}!".format(contest),
            )

            # Confirm there aren't multiple Candidacies with the same person_id
            person_id_groups_q = contest.candidacies.values(
                'person_id',
            ).annotate(
                row_count=models.Count('id'),
            ).order_by().filter(row_count__gt=1)

            self.assertTrue(
                person_id_groups_q.count() == 0,
                msg="Multiple candidacies with same person_id in {}!".format(
                    contest
                ),
            )

            # Confirm there aren't multiple Candidacies with the same filer_id
            filer_id_groups_q = contest.candidacies.filter(
                person__identifiers__scheme='calaccess_filer_id'
            ).values(
                'person__identifiers__identifier'
            ).annotate(
                row_count=models.Count('id'),
            ).order_by().filter(row_count__gt=1)

            self.assertTrue(
                filer_id_groups_q.count() == 0,
                msg="Multiple candidacies with same filer_id in {}!".format(
                    contest
                ),
            )

            # Confirm there aren't multiple Candidacies with the same candidate_name
            # unless party_id is different or person's filer_id is different
            candidate_name_groups_q = contest.candidacies.values(
                'candidate_name',
            ).annotate(
                row_count=models.Count('id'),
            ).order_by().filter(row_count__gt=1)

            # loop over each group of multiple candidacies sharing the same candidate_name
            for group in candidate_name_groups_q.all():
                candidacies_q = contest.candidacies.filter(
                    candidate_name=group['candidate_name'],
                )

                filer_id_party_groups_q = candidacies_q.filter(
                    person__identifiers__scheme='calaccess_filer_id'
                ).order_by(
                    'person__identifiers__identifier',
                    'party',
                ).values(
                    'person__identifiers__identifier',
                    'party',
                ).distinct()

                # confirm that count of candidacies equals count of
                # distinct filer_id/party groups
                self.assertEqual(
                    candidacies_q.count(),
                    filer_id_party_groups_q.count(),
                    msg='{0} candidacies in {1} with candidate_name "{2}" have {3} '
                        'distinct filer_id/party combos!'.format(
                        candidacies_q.count(),
                        contest,
                        group['candidate_name'],
                        filer_id_party_groups_q.count(),
                    ),
                )

            # Confirm there aren't multiple Candidacies with the same person.name
            # unless party_id is different or person's filer_id is different
            person_name_groups_q = contest.candidacies.values(
                'person__name',
            ).annotate(
                row_count=models.Count('id'),
            ).order_by().filter(row_count__gt=1)

            # loop over each group of multiple candidacies sharing the same candidate_name
            for group in person_name_groups_q.all():
                candidacies_q = contest.candidacies.filter(
                    person__name=group['person__name'],
                )

                filer_id_party_groups_q = candidacies_q.filter(
                    person__identifiers__scheme='calaccess_filer_id'
                ).order_by(
                    'person__identifiers__identifier',
                    'party',
                ).values(
                    'person__identifiers__identifier',
                    'party',
                ).distinct()

                # confirm that count of candidacies equals count of
                # distinct filer_id/party groups
                self.assertEqual(
                    candidacies_q.count(),
                    filer_id_party_groups_q.count(),
                    msg='{0} candidacies in {1} with person__name "{2}" have {3} '
                        'distinct filer_id/party combos!'.format(
                        candidacies_q.count(),
                        contest,
                        group['person__name'],
                        filer_id_party_groups_q.count(),
                    ),
                )

#        # For each Person...
#        for person in Person.objects.all():
#            # Confirm name is same as most recent candidate_name
#            latest_candidate_name = person.candidacies.latest(
#                'contest__election__date'
#            ).candidate_name

#            self.assertEqual(
#                person.name,
#                latest_candidate_name,
#                msg='Person.name "{0}" doesn\'t match latest candidate_name "{1}!'.format(
#                    person.name,
#                    latest_candidate_name,
#                )
#            )

    # def test_form460_filings_count(self):
    #     """
    #     Confirm count of Form460Filing is equal to OCD Filing count.
    #     """
    #     passed, error_msg = VerifyCmd().test_form460_filings_count()
    #
    #     self.assertTrue(passed, error_msg)
    #
    # def test_form460_filing_actions_count(self):
    #     """
    #     Confirm count of Form460FilingVersion is equal to OCD FilingAction count.
    #     """
    #     passed, error_msg = VerifyCmd().test_form460_filing_actions_count()
    #
    #     self.assertTrue(passed, error_msg)
    # def test_regular_assembly_contest_counts(self):
    #     """
    #     Confirm equality of actual and expected counts of assembly contests in each election.
    #
    #     Test only prior elections.
    #     """
    #     passed, error_msg = VerifyCmd().test_regular_assembly_contest_counts()
    #
    #     self.assertTrue(passed, error_msg)
    #
    # def test_regular_executive_contest_counts(self):
    #     """
    #     Confirm equality of actual and expected counts of senate contests in each election.
    #
    #     Test only prior elections.
    #     """
    #     passed, error_msg = VerifyCmd().test_regular_executive_contest_counts()
    #
    #     self.assertTrue(passed, error_msg)
    #
    # def test_regular_senate_contest_counts(self):
    #     """
    #     Confirm equality of actual and expected counts of senate contests in each election.
    #
    #     Test only prior elections.
    #     """
    #     passed, error_msg = VerifyCmd().test_regular_senate_contest_counts()
    #
    #     self.assertTrue(passed, error_msg)
    #
    # def test_regular_senate_contest_districts(self):
    #     """
    #     Confirm that no elections have senate contests in the wrong districts.
    #     """
    #     passed, error_msg = VerifyCmd().test_regular_senate_contest_districts()
    #
    #     self.assertTrue(passed, error_msg)
    #
    # def test_for_duplicate_memberships(self):
    #     """
    #     Confirm there are no duplicate membership records.
    #     """
    #     passed, error_msg = VerifyCmd().test_for_duplicate_memberships()
    #
    #     self.assertTrue(passed, error_msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals

# Models
from calaccess_processed.proxies import OCDProxyModelMixin
from opencivicdata.core.models import Person, PersonIdentifier, PersonName

# Managers
from calaccess_processed.managers import BulkLoadSQLManager
from calaccess_processed_elections.managers import OCDPersonManager


class OCDPersonProxy(Person, OCDProxyModelMixin):
    """
    A proxy on the OCD Person model with helper methods.
    """
    objects = OCDPersonManager()

    copy_to_fields = (
        ('id',),
        ('name',),
        ('sort_name',),
        ('family_name',),
        ('given_name',),
        ('image',),
        ('gender',),
        ('summary',),
        ('national_identity',),
        ('biography',),
        ('birth_date',),
        ('death_date',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True

    def update_name(self):
        """
        Update name field to the latest candidate record.
        """
        # Get the latest candidate name
        latest_candidate_name = self.candidacies.latest('contest__election__date').candidate_name
        # If the latest candidate name doesn't match the current name
        if self.name != latest_candidate_name:
            # Move the current name into other_names
            if not self.other_names.filter(name=self.name).exists():
                self.other_names.create(name=self.name)
            # Reset the main one
            self.name = latest_candidate_name
            # Save out.
            self.save()

    def add_other_name(self, name, note):
        """
        Add name to Person's list of other names along with note.

        If name was previously added to person, append note to existing note.

        Return boolean value indicating if the name was created or not.
        """
        # If the provided name is the default name, just quit.
        if name == self.name:
            created = False
        elif self.other_names.filter(name=name).exists():
            created = False
            # append the note, if not already there
            existing_name = self.other_names.get(name=name)
            if note not in existing_name.note:
                existing_name.note = '{0}, {1}'.format(existing_name.note, note)
        else:
            created = True
            self.other_names.create(name=name, note=note)
        return created

    def add_filer_id(self, filer_id):
        """
        Adds the provided CAL-ACCESS filer_id to the object metadata.
        """
        kwargs = dict(
            scheme="calaccess_filer_id",
            identifier=filer_id
        )
        # If it already exists, quit out.
        if self.identifiers.filter(**kwargs).count():
            return False
        # Otherwise add it.
        self.identifiers.create(**kwargs)
        return True

    @property
    def filer_id(self):
        """
        Returns the CAL-ACCESS filer_id linked with the object, if any.
        """
        return self.identifiers.get(scheme="calaccess_filer_id")

    @property
    def scraped_candidates(self):
        """
        Returns the scraped candidates linked to this candidacy.
        """
        from calaccess_processed_elections.proxies import ScrapedCandidateProxy
        filer_ids = [i.identifier for i in self.identifiers.filter(scheme="calaccess_filer_id")]
        return ScrapedCandidateProxy.objects.filter(scraped_id__in=filer_ids).order_by("-election")

    @property
    def form501s(self):
        """
        Returns any linked Form 501s filings.
        """
        from calaccess_processed_elections.proxies import OCDCandidacyProxy
        candidacies = OCDCandidacyProxy.objects.filter(person=self)
        form501s = []
        for c in candidacies:
            for f in c.form501s:
                if f not in form501s:
                    form501s.append(f)
        return form501s


class OCDPersonIdentifierProxy(PersonIdentifier, OCDProxyModelMixin):
    """
    A proxy on the OCD PersonIdentifier model.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDPersonNameProxy(PersonName, OCDProxyModelMixin):
    """
    A proxy on the OCD PersonName model with helper methods.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD Filing related models..
"""
from __future__ import unicode_literals
from opencivicdata.campaign_finance.models import (
    Filing,
    FilingAction,
    FilingActionSummaryAmount,
    FilingIdentifier,
    FilingSource
)
from calaccess_processed.proxies import OCDProxyModelMixin
from calaccess_processed_campaignfinance.managers import (
    CampaignFinanceManager,
    OCDFilingManager,
    OCDFilingIdentifierManager,
    OCDFilingActionManager,
    OCDFilingActionSummaryAmountManager
)

# Logging
import logging
logger = logging.getLogger(__name__)


class OCDFilingProxy(Filing, OCDProxyModelMixin):
    """
    A proxy on the OCD Filing model.
    """
    objects = OCDFilingManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True

    @property
    def current_action(self):
        """
        Returns the most current action linked with this filing.
        """
        return self.actions.get(is_current=True)

    @property
    def calaccess_filing_id(self):
        """
        Returns the filing's CAL-ACCESS filing ID.
        """
        return self.identifiers.get(scheme="calaccess_filing_id")

    @property
    def calaccess_amend_id(self):
        """
        Returns the filing's CAL-ACCESS amendment id.
        """
        return self.current_action.extras['amend_id']

    @property
    def calaccess_filing_url(self):
        """
        Returns the filing's URL on the CAL-ACCESS website.
        """
        url_template = "http://cal-access.sos.ca.gov/PDFGen/pdfgen.prg?filingid={}&amendid={}"
        return url_template.format(
            self.calaccess_filing_id.identifier,
            self.calaccess_amend_id
        )

    @property
    def calaccess_filer_id(self):
        """
        Returns the filer's CAL-ACCESS filer id.
        """
        return self.filer.identifiers.get(scheme="calaccess_filer_id")

    @property
    def calaccess_filer_url(self):
        """
        Returns the URL of the filer's detail page on the CAL-ACCESS website.
        """
        url_template = "http://cal-access.sos.ca.gov/Campaign/Committees/Detail.aspx?id={}"
        return url_template.format(self.calaccess_filer_id.identifier)


class OCDFilingIdentifierProxy(FilingIdentifier, OCDProxyModelMixin):
    """
    A proxy on the OCD FilingIdentifier model.
    """
    objects = OCDFilingIdentifierManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDFilingSourceProxy(FilingSource, OCDProxyModelMixin):
    """
    A proxy on the OCD FilingSource model.
    """
    objects = CampaignFinanceManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDFilingActionProxy(FilingAction, OCDProxyModelMixin):
    """
    A proxy on the OCD FilingAction model.
    """
    objects = OCDFilingActionManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True


class OCDFilingActionSummaryAmountProxy(FilingActionSummaryAmount, OCDProxyModelMixin):
    """
    A proxy on the OCD FilingActionSummaryAmount model.
    """
    objects = OCDFilingActionSummaryAmountManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_campaignfinance"
        proxy = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for OCD Committee related managers.
"""
from __future__ import unicode_literals
import logging
from .base import CampaignFinanceManager
logger = logging.getLogger(__name__)


class OCDCommitteeManager(CampaignFinanceManager):
    """
    Manager with custom methods for OCD Committee model.
    """
    def load(self):
        """
        Load OCD Committee with data extracted from Form460Filing.
        """
        # Updating name of existing committees..
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/update_committee_names_from_form460s')
        # ... and inserting new committees.
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/insert_committees_from_form460s')


class OCDCommitteeIdentifierManager(CampaignFinanceManager):
    """
    Manager with custom methods for OCD CommitteeIdentifier model.
    """
    def load(self):
        """
        Load OCD CommitteeIdentifier with data extracted from Form460Filing.
        """
        #  Insert new committee identifiers.
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/insert_committee_ids_from_form460s')
        # Removing "calaccess_filer_id" from extras field on Committee.
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/remove_calaccess_filer_ids_from_committees')


class OCDCommitteeNameManager(CampaignFinanceManager):
    """
    Manager with custom methods for OCD CommitteeName model.
    """
    def load(self):
        """
        Load OCD CommitteeName with data extracted from Form460Filing.
        """
        # Insert new committee names
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/insert_committee_names_from_form460s')
        # Delete current names from extra list of committee names.
        self.execute_custom_sql('opencivicdata/campaign_finance/committees/delete_current_names_from_committee_names')


class OCDCommitteeTypeManager(CampaignFinanceManager):
    """
    Manager with custom methods for OCD CommitteeType model.
    """
    def load(self):
        """
        Seeds the default data for this model.
        """
        from calaccess_processed_elections.proxies import OCDJurisdictionProxy

        # Prep data
        qs = self.get_queryset()
        jurisdiction = OCDJurisdictionProxy.objects.california()

        # Fire away
        qs.get_or_create(name='Recipient', jurisdiction=jurisdiction)
        qs.get_or_create(name='Candidate', jurisdiction=jurisdiction)
        qs.get_or_create(name='Ballot Measure', jurisdiction=jurisdiction)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
General utilities for the application.
"""
from __future__ import unicode_literals
default_app_config = 'calaccess_processed.apps.CalAccessProcessedConfig'


def archive_directory_path(instance, filename):
    """
    Returns a path to an archived processed data file or ZIP.
    """
    from .models import ProcessedDataZip, ProcessedDataFile

    if isinstance(instance, ProcessedDataZip):
        release_datetime = instance.version.raw_version.release_datetime
        f_name, f_ext = filename.split('.')
        return '{fn}_{dt:%Y-%m-%d_%H-%M-%S}.{fx}'.format(
            fn=f_name,
            dt=release_datetime,
            fx=f_ext,
        )
    elif isinstance(instance, ProcessedDataFile):
        release_datetime = instance.version.raw_version.release_datetime
        return '{dt:%Y-%m-%d_%H-%M-%S}/{f}'.format(dt=release_datetime, f=filename)
    else:
        raise TypeError("Must be ProcessedDataVersion or ProcessedDataFile instance.")


__all__ = ('archive_directory_path',)
